@@ -23,13 +23,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define RRR_CMODULE_EXT_H
 
 struct rrr_cmodule_worker;
-struct rrr_message;
-struct rrr_message_addr;
+struct rrr_msg_msg;
+struct rrr_msg_addr;
 
 int rrr_cmodule_ext_send_message_to_parent (
 		struct rrr_cmodule_worker *worker,
-		struct rrr_message *message,
-		const struct rrr_message_addr *message_addr
+		struct rrr_msg_msg *message,
+		const struct rrr_msg_addr *message_addr
 );
 
 #endif /* RRR_CMODULE_EXT_H */
