@@ -33,16 +33,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../rrr_strerror.h"
 #include "../rrr_mmap.h"
 #include "../mmap_channel.h"
-#include "../message_addr.h"
-#include "../message_log.h"
-#include "../messages.h"
-#include "../rrr_time.h"
-#include "../posix.h"
+#include "../messages/msg_addr.h"
+#include "../messages/msg_log.h"
+#include "../messages/msg_msg.h"
 #include "../fork.h"
 #include "../common.h"
-#include "../ip_buffer_entry.h"
-#include "../gnu.h"
-#include "../macro_utils.h"
+#include "../message_holder/message_holder.h"
+#include "../util/gnu.h"
+#include "../util/macro_utils.h"
+#include "../util/rrr_time.h"
+#include "../util/posix.h"
 
 #define ALLOCATE_TMP_NAME(target, name1, name2)															\
 	if (rrr_asprintf(&target, "%s-%s", name1, name2) <= 0) {											\
@@ -76,12 +76,12 @@ static int __rrr_cmodule_worker_new (
 
 	memset(worker, '\0', sizeof(*worker));
 
-	if ((rrr_mmap_channel_new(&worker->channel_to_fork, cmodule->mmap, name)) != 0) {
+	if ((ret = rrr_mmap_channel_new(&worker->channel_to_fork, cmodule->mmap, name)) != 0) {
 		RRR_MSG_0("Could not create mmap channel in __rrr_cmodule_worker_new\n");
 		goto out_free;
 	}
 
-	if ((rrr_mmap_channel_new(&worker->channel_to_parent, cmodule->mmap, name)) != 0) {
+	if ((ret = rrr_mmap_channel_new(&worker->channel_to_parent, cmodule->mmap, name)) != 0) {
 		RRR_MSG_0("Could not create mmap channel in __rrr_cmodule_worker_new\n");
 		goto out_destroy_channel_to_fork;
 	}
@@ -92,7 +92,7 @@ static int __rrr_cmodule_worker_new (
 		goto out_destroy_channel_to_parent;
 	}
 
-	if ((pthread_mutex_init(&worker->pid_lock, NULL)) != 0) {
+	if ((rrr_posix_mutex_init(&worker->pid_lock, 0)) != 0) {
 		RRR_MSG_0("Could not initialize lock in __rrr_cmodule_worker_new\n");
 		ret = 1;
 		goto out_free_name;
@@ -219,8 +219,8 @@ struct rrr_cmodule_process_callback_data {
 static int __rrr_cmodule_worker_loop_read_callback (const void *data, size_t data_size, void *arg) {
 	struct rrr_cmodule_process_callback_data *callback_data = arg;
 
-	const struct rrr_message *msg = data;
-	const struct rrr_message_addr *msg_addr = data + MSG_TOTAL_SIZE(msg);
+	const struct rrr_msg_msg *msg = data;
+	const struct rrr_msg_addr *msg_addr = data + MSG_TOTAL_SIZE(msg);
 
 	if (MSG_TOTAL_SIZE(msg) + sizeof(*msg_addr) != data_size) {
 		RRR_BUG("BUG: Size mismatch in __rrr_cmodule_worker_loop_read_callback %i+%lu != %lu\n",
@@ -255,9 +255,9 @@ static int __rrr_cmodule_worker_spawn_message (
 ) {
 	int ret = 0;
 
-	struct rrr_message *message = NULL;
+	struct rrr_msg_msg *message = NULL;
 
-	if (rrr_message_new_empty (
+	if (rrr_msg_msg_new_empty (
 			&message,
 			MSG_TYPE_MSG,
 			MSG_CLASS_DATA,
@@ -271,8 +271,8 @@ static int __rrr_cmodule_worker_spawn_message (
 		goto out;
 	}
 
-	struct rrr_message_addr message_addr;
-	rrr_message_addr_init(&message_addr);
+	struct rrr_msg_addr message_addr;
+	rrr_msg_addr_init(&message_addr);
 
 	if ((ret = process_callback(
 			worker,
@@ -309,7 +309,7 @@ static int __rrr_cmodule_worker_loop (
 	};
 
 	// Control stuff
-	uint64_t time_now = rrr_time_get_64();
+	uint64_t time_now = 0;
 	uint64_t next_spawn_time = 0;
 
 	if (worker->config_data->sleep_time_us > worker->config_data->spawn_interval_us) {
@@ -326,8 +326,6 @@ static int __rrr_cmodule_worker_loop (
 	uint64_t prev_stats_time = 0;
 
 	while (worker->received_stop_signal == 0) {
-		// Check for backlog on the socket. Don't process any more messages untill backlog is cleared up
-
 		time_now = rrr_time_get_64();
 
 		if (next_spawn_time == 0) {
@@ -438,8 +436,8 @@ int rrr_cmodule_worker_loop_start (
 		goto out;
 	}
 
-	struct rrr_socket_msg control_msg = {0};
-	rrr_socket_msg_populate_control_msg(&control_msg, RRR_CMODULE_CONTROL_MSG_CONFIG_COMPLETE, 1);
+	struct rrr_msg control_msg = {0};
+	rrr_msg_populate_control_msg(&control_msg, RRR_CMODULE_CONTROL_MSG_CONFIG_COMPLETE, 1);
 
 	if (rrr_mmap_channel_write(
 			worker->channel_to_parent,
@@ -498,7 +496,7 @@ static int __rrr_cmodule_worker_signal_handler (int signal, void *private_arg) {
 	struct rrr_cmodule_worker *worker = private_arg;
 
 	if (signal == SIGUSR1 || signal == SIGINT || signal == SIGTERM) {
-		RRR_DBG_SIGNAL("script worker %s pid %i received SIGUSR1, SIGTERM or SIGINT, stopping\n",
+		RRR_DBG_SIGNAL("cmodule worker %s pid %i received SIGUSR1, SIGTERM or SIGINT, stopping\n",
 				worker->name, getpid());
 		worker->received_stop_signal = 1;
 	}
@@ -514,23 +512,26 @@ static void __rrr_cmodule_worker_fork_log_hook (
 ) {
 	struct rrr_cmodule_worker *worker = private_arg;
 
-	struct rrr_message_log *message_log = NULL;
+	struct rrr_msg_log *message_log = NULL;
 
-	if (rrr_message_log_new(&message_log, loglevel_translated, prefix, message) != 0) {
+	if (rrr_msg_msg_log_new(&message_log, loglevel_translated, prefix, message) != 0) {
 		goto out;
 	}
 
 	int ret = 0;
-	if ((ret = rrr_mmap_channel_write(
+	if ((ret = rrr_mmap_channel_write (
 			worker->channel_to_parent,
 			message_log,
 			message_log->msg_size,
 			RRR_CMODULE_CHANNEL_WAIT_TIME_US
 	)) != 0) {
 		if (ret == RRR_MMAP_CHANNEL_FULL) {
-			RRR_MSG_0("mmap channel was full in __rrr_cmodule_worker_fork_log_hook for worker %s\n",
-					worker->name);
-			ret = 1;
+			RRR_MSG_0("Warning: mmap channel was full in __rrr_cmodule_worker_fork_log_hook for worker %s in log hook\n",
+				worker->name);
+		}
+		else {
+			RRR_MSG_0("Warning: Error %i while writing to mmap channel in __rrr_cmodule_worker_fork_log_hook for worker %s in log hook\n",
+				ret, worker->name);
 		}
 	}
 
@@ -621,13 +622,21 @@ int rrr_cmodule_worker_fork_start (
 
 	int was_found = 0;
 
-	// Preserve fork signal andler in case child makes any forks
-	rrr_signal_handler_remove_all_except(&was_found, &rrr_fork_signal_handler);
-	if (was_found == 0) {
-		RRR_BUG("BUG: rrr_fork_signal_handler was not registered in rrr_cmodule_worker_fork_start, should have been added in main()\n");
-	}
+	{
+		// There is no guarantee for whether signals are active or not at this point. Disable
+		// signals while working with the handler list, then always set ACTIVE afterwards.
+		rrr_signal_handler_set_active(RRR_SIGNALS_NOT_ACTIVE);
 
-	rrr_signal_handler_push(__rrr_cmodule_worker_signal_handler, worker);
+		// Preserve fork signal andler in case child makes any forks
+		rrr_signal_handler_remove_all_except(&was_found, &rrr_fork_signal_handler);
+		if (was_found == 0) {
+			RRR_BUG("BUG: rrr_fork_signal_handler was not registered in rrr_cmodule_worker_fork_start, should have been added in main()\n");
+		}
+
+		rrr_signal_handler_push(__rrr_cmodule_worker_signal_handler, worker);
+
+		rrr_signal_handler_set_active(RRR_SIGNALS_ACTIVE);
+	}
 
 	// It's safe to use the char * from cmodule_data. It will never
 	// get freed by the fork, instances framework does that when the thread is exiting.
