@@ -19,11 +19,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
-#include "settings.h"
-
 #ifndef RRR_CONFIGURATION_H
 #define RRR_CONFIGURATION_H
 
+#include "settings.h"
+#include "array_tree.h"
+
 #define RRR_CONFIG_MAX_MODULES CMD_MAXIMUM_CMDLINE_ARGS
 #define RRR_CONFIG_MAX_SIZE 16*1024*1024
 #define RRR_CONFIG_MAX_SETTINGS 32
@@ -32,10 +33,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 struct rrr_config {
 	int module_count;
 	int module_count_max;
-	struct rrr_instance_config **configs;
+	struct rrr_instance_config_data **configs;
+	struct rrr_array_tree_list array_trees;
 };
 
-struct rrr_instance_config *rrr_config_find_instance (struct rrr_config *source, const char *name);
+struct rrr_instance_config_data *rrr_config_find_instance (struct rrr_config *source, const char *name);
 void rrr_config_destroy (struct rrr_config *target);
 struct rrr_config *rrr_config_parse_file (const char *filename);
 int rrr_config_dump (struct rrr_config *config);
