@@ -21,6 +21,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h>
 #include <pthread.h>
 
 #include "../log.h"
@@ -30,7 +31,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "http_util.h"
 
 #include "../threads.h"
-#include "../macro_utils.h"
+#include "../util/macro_utils.h"
 
 #define RRR_HTTP_PART_DECLARE_DATA_START_AND_END(part,data_ptr)	\
 		const char *data_start =								\
@@ -46,7 +47,7 @@ static int __rrr_http_part_check_content_type (
 		struct rrr_http_part *part,
 		const char *content_type_test
 ) {
-	const struct rrr_http_header_field *content_type = rrr_http_part_get_header_field(part, "content-type");
+	const struct rrr_http_header_field *content_type = rrr_http_part_header_field_get(part, "content-type");
 	if (content_type == NULL || content_type->value == NULL || *(content_type->value) == '\0') {
 		return 0;
 	}
@@ -201,9 +202,9 @@ static int __rrr_http_header_parse_content_type_value (RRR_HTTP_HEADER_FIELD_PAR
 		int found = 0;
 		const char *unquote_field_names[] = {"charset", "boundary"};
 		__rrr_http_header_parse_unquote_fields(&found, node, field->name, unquote_field_names, 2);
-		if (found == 0) {
+/*		if (found == 0) {
 			RRR_DBG_1("Warning: Unknown field '%s' in content-type header\n", node->name);
-		}
+		}*/
 	RRR_LL_ITERATE_END();
 
 	out:
@@ -348,12 +349,12 @@ static int __rrr_http_header_field_new (
 	return ret;
 }
 
-const struct rrr_http_field *__rrr_http_part_get_header_field_subvalue (
+const struct rrr_http_field *__rrr_http_part_header_field_subvalue_get (
 		const struct rrr_http_part *part,
 		const char *field_name,
 		const char *subvalue_name
 ) {
-	const struct rrr_http_header_field *field = rrr_http_part_get_header_field(part, field_name);
+	const struct rrr_http_header_field *field = rrr_http_part_header_field_get(part, field_name);
 	if (field == NULL) {
 		return NULL;
 	}
@@ -372,6 +373,7 @@ void rrr_http_part_destroy (struct rrr_http_part *part) {
 	RRR_LL_DESTROY(&part->chunks, struct rrr_http_chunk, free(node));
 	rrr_http_field_collection_clear(&part->fields);
 	RRR_FREE_IF_NOT_NULL(part->response_str);
+	RRR_FREE_IF_NOT_NULL(part->response_raw_data);
 	RRR_FREE_IF_NOT_NULL(part->request_uri);
 	RRR_FREE_IF_NOT_NULL(part->request_method_str);
 	free (part);
@@ -409,7 +411,29 @@ int rrr_http_part_new (struct rrr_http_part **result) {
 	return ret;
 }
 
-const struct rrr_http_header_field *rrr_http_part_get_header_field (
+void rrr_http_part_set_allocated_raw_response (
+		struct rrr_http_part *part,
+		char **raw_data_source,
+		size_t raw_data_size
+) {
+	if (part->response_raw_data != NULL) {
+		RRR_BUG("BUG: rrr_http_part_set_allocated_raw_response called while raw data was already set\n");
+	}
+	part->response_raw_data = *raw_data_source;
+	part->response_raw_data_size = raw_data_size;
+	*raw_data_source = NULL;
+}
+
+void rrr_http_part_set_raw_request_ptr (
+		struct rrr_http_part *part,
+		const char *raw_data,
+		size_t raw_data_size
+) {
+	part->request_raw_data = raw_data;
+	part->request_raw_data_size = raw_data_size;
+}
+
+const struct rrr_http_header_field *rrr_http_part_header_field_get (
 		const struct rrr_http_part *part,
 		const char *name_lowercase
 ) {
@@ -468,7 +492,7 @@ static int __rrr_http_parse_response_code (
 	const char *start_orig = start;
 
 	rrr_length tmp_len = 0;
-	if ((ret = rrr_http_util_strcasestr(&start, &tmp_len, start, crlf, "HTTP/1.1")) != 0 || start != start_orig) {
+	if (rrr_http_util_strcasestr(&start, &tmp_len, start, crlf, "HTTP/1.1") != 0 || start != start_orig) {
 		RRR_MSG_0("Could not understand HTTP response header/version in __rrr_http_parse_response_code\n");
 		ret = RRR_HTTP_PARSE_SOFT_ERR;
 		goto out;
@@ -478,7 +502,7 @@ static int __rrr_http_parse_response_code (
 	start += rrr_http_util_count_whsp(start, end);
 
 	unsigned long long int response_code = 0;
-	if ((ret = rrr_http_util_strtoull(&response_code, &tmp_len, start, crlf, 10)) != 0 || response_code > 999) {
+	if (rrr_http_util_strtoull(&response_code, &tmp_len, start, crlf, 10) != 0 || response_code > 999) {
 		RRR_MSG_0("Could not understand HTTP response code in __rrr_http_parse_response_code\n");
 		ret = RRR_HTTP_PARSE_SOFT_ERR;
 		goto out;
@@ -571,6 +595,15 @@ static int __rrr_http_parse_request (
 
 	rrr_length protocol_length = 0;
 	if ((ret = rrr_http_util_strcasestr(&start, &protocol_length, start, crlf, "HTTP/1.1")) != 0 || start != start_orig) {
+		if (	rrr_http_util_strcasestr(&start, &protocol_length, start, crlf, "HTTP/0.9") ||
+				rrr_http_util_strcasestr(&start, &protocol_length, start, crlf, "HTTP/1.0") ||
+				rrr_http_util_strcasestr(&start, &protocol_length, start, crlf, "HTTP/2.0")
+		) {
+			result->response_code = RRR_HTTP_RESPONSE_CODE_VERSION_NOT_SUPPORTED;
+		}
+		else {
+			result->response_code = RRR_HTTP_RESPONSE_CODE_ERROR_BAD_REQUEST;
+		}
 		RRR_MSG_0("Invalid or missing protocol version in HTTP request\n");
 		ret = RRR_HTTP_PARSE_SOFT_ERR;
 		goto out;
@@ -733,179 +766,182 @@ static int __rrr_http_parse_header_field (
 		const char *end
 ) {
 	int ret = 0;
-	struct rrr_http_header_field *field = NULL;
 	const char *start = start_orig;
+	struct rrr_http_header_field_collection fields_tmp = {0};
+	struct rrr_http_header_field *field = NULL;
 
 	*parsed_bytes = 0;
 
-	// NOTE : Be very careful with choosing to parse up to CRLF or END.
-	//        We sometimes search for content up to CRLF and sometimes END (allowing CRLF in between).
+	while (1) {
+		int bad_client_missing_space_after_comma = 0;
 
-	const char *crlf = NULL;
-	int bad_client_missing_space_after_comma = 0;
+		// NOTE : Be very careful with choosing to parse up to CRLF or END.
+		//        We sometimes search for content up to CRLF and sometimes END (allowing CRLF in between).
 
-	// >>>> Duplicate field goto target
-	do_duplicate_field:
+		const char *crlf = NULL;
 
-	bad_client_missing_space_after_comma = 0;
-
-	crlf = rrr_http_util_find_crlf(start, end);
-	if (crlf == NULL) {
-		ret = RRR_HTTP_PARSE_INCOMPLETE;
-		goto out;
-	}
-
-	if (field == NULL) {
-		const char *colon = rrr_http_util_strchr(start, crlf, ':');
-		if (colon == NULL) {
-			RRR_MSG_0("Colon not found in HTTP header field in __rrr_http_parse_header_field\n");
-			rrr_http_util_print_where_message(start, end);
-			ret = RRR_HTTP_PARSE_SOFT_ERR;
-			goto out;
-		}
-
-		if ((ret = __rrr_http_header_field_new(&field, start, colon - start)) != 0) {
-			ret = RRR_HTTP_PARSE_HARD_ERR;
-			goto out;
-		}
-
-		start = colon + 1;
-
-		RRR_DBG_3("parsing field with name: %s\n", field->name);
-	}
-	else {
-		const char *old_name = field->name;
-		size_t old_name_length = strlen(field->name);
-
-		// Field is already added to list, make sure it is not freed if we get an error
-		field = NULL;
-
-		if ((ret = __rrr_http_header_field_new(&field, old_name, old_name_length)) != 0) {
-			ret = RRR_HTTP_PARSE_HARD_ERR;
-			goto out;
-		}
-
-		if (*start == ',') {
-			start++;
-		}
-
-		if (start >= end) {
+		crlf = rrr_http_util_find_crlf(start, end);
+		if (crlf == NULL) {
 			ret = RRR_HTTP_PARSE_INCOMPLETE;
 			goto out;
 		}
 
-		if (rrr_http_util_count_whsp(start, end) == 0) {
-			bad_client_missing_space_after_comma = 1;
-		}
-
-		RRR_DBG_3("parsing duplicate field with name: %s\n", field->name);
-	}
-
-	if (start >= crlf) {
-		RRR_MSG_0("No value for header field in __rrr_http_parse_header_field\n");
-		ret = RRR_HTTP_PARSE_SOFT_ERR;
-		goto out;
-	}
-
-	ssize_t whitespace_count = rrr_http_util_count_whsp(start, crlf);
-
-	if (start + whitespace_count == crlf) {
-		// Continue on next line
-		start = crlf + 2;
-		crlf = NULL;
-	}
-	else if (whitespace_count == 0) {
-		if (bad_client_missing_space_after_comma) {
-			if (RRR_DEBUGLEVEL_1) {
-				RRR_MSG_0("Warning: No whitespace after comma while parsing HTTP header field subvalues\n");
+		if (RRR_LL_COUNT(&fields_tmp) == 0) {
+			const char *colon = rrr_http_util_strchr(start, crlf, ':');
+			if (colon == NULL) {
+				RRR_MSG_0("Colon not found in HTTP header field in __rrr_http_parse_header_field\n");
 				rrr_http_util_print_where_message(start, end);
+				ret = RRR_HTTP_PARSE_SOFT_ERR;
+				goto out;
 			}
+
+			if (__rrr_http_header_field_new(&field, start, colon - start) != 0) {
+				ret = RRR_HTTP_PARSE_HARD_ERR;
+				goto out;
+			}
+
+			start = colon + 1;
+
+			RRR_DBG_3("parsing field with name: %s\n", field->name);
 		}
 		else {
-			RRR_MSG_0("Error: No whitespace after separator while parsing HTTP header field subvalues\n");
-			ret = RRR_HTTP_PARSE_SOFT_ERR;
-			goto out;
-		}
-	}
+			// Duplicate field (after comma)
+			const char *last_field_name = RRR_LL_LAST(&fields_tmp)->name;
 
-	crlf = rrr_http_util_find_crlf(start, end);
-	if (crlf == NULL) {
-		ret = RRR_HTTP_PARSE_INCOMPLETE;
-		goto out;
-	}
+			if (__rrr_http_header_field_new(&field, last_field_name, strlen(last_field_name)) != 0) {
+				ret = RRR_HTTP_PARSE_HARD_ERR;
+				goto out;
+			}
 
-	int prev_subvalue_count = 0;
-	int comma_found_do_duplicate_field = 0;
+			if (*start == ',') {
+				start++;
+			}
 
-	do {
-		prev_subvalue_count = RRR_LL_COUNT(&field->fields);
-
-		ssize_t parsed_bytes_tmp = 0;
-		RRR_DBG_3("subvalue start: %c\n", *start);
-		if ((ret = __rrr_http_parse_header_field_subvalue (
-				&field->fields,
-				&parsed_bytes_tmp,
-				start,
-				end,
-				(field->definition != NULL ? field->definition->flags : 0),
-				bad_client_missing_space_after_comma
-		)) != 0) {
-			goto out;
-		}
-		start += parsed_bytes_tmp;
-
-		bad_client_missing_space_after_comma = 0;
-
-		if (*start == ';') {
-			const char *next = start + 1;
-			if (next > end) {
+			if (start >= end) {
 				ret = RRR_HTTP_PARSE_INCOMPLETE;
 				goto out;
 			}
-			if (*next != ' ' && *next != '\t' && *next != '\r') {
+
+			if (rrr_http_util_count_whsp(start, end) == 0) {
 				bad_client_missing_space_after_comma = 1;
 			}
-			if (*next == ',') {
+
+			RRR_DBG_3("parsing duplicate field with name: %s\n", field->name);
+		}
+
+		if (start >= crlf) {
+			RRR_MSG_0("No value for header field in __rrr_http_parse_header_field\n");
+			ret = RRR_HTTP_PARSE_SOFT_ERR;
+			goto out;
+		}
+
+		ssize_t whitespace_count = rrr_http_util_count_whsp(start, crlf);
+
+		if (start + whitespace_count == crlf) {
+			// Continue on next line
+			start = crlf + 2;
+			crlf = NULL;
+		}
+		else if (whitespace_count == 0) {
+			if (bad_client_missing_space_after_comma) {
 				if (RRR_DEBUGLEVEL_1) {
-					RRR_MSG_0("Warning: Comma found after semicolon in HTTP header, bad implementation\n");
+					RRR_MSG_0("Warning: No whitespace after comma while parsing HTTP header field subvalues\n");
 					rrr_http_util_print_where_message(start, end);
 				}
-				start++;
+			}
+			else {
+				RRR_MSG_0("Error: No whitespace after separator while parsing HTTP header field subvalues\n");
+				ret = RRR_HTTP_PARSE_SOFT_ERR;
+				goto out;
 			}
 		}
 
-		if (*start == ',') {
-			comma_found_do_duplicate_field = 1;
+		crlf = rrr_http_util_find_crlf(start, end);
+		if (crlf == NULL) {
+			ret = RRR_HTTP_PARSE_INCOMPLETE;
+			goto out;
+		}
+
+		int prev_subvalue_count = 0;
+		int comma_found_do_duplicate_field = 0;
+
+		do {
+			prev_subvalue_count = RRR_LL_COUNT(&field->fields);
+
+			ssize_t parsed_bytes_tmp = 0;
+			RRR_DBG_3("subvalue start: %c\n", *start);
+			if ((ret = __rrr_http_parse_header_field_subvalue (
+					&field->fields,
+					&parsed_bytes_tmp,
+					start,
+					end,
+					(field->definition != NULL ? field->definition->flags : 0),
+					bad_client_missing_space_after_comma
+			)) != 0) {
+				goto out;
+			}
+			start += parsed_bytes_tmp;
+
+			bad_client_missing_space_after_comma = 0;
+
+			if (start >= end) {
+				ret = RRR_HTTP_PARSE_INCOMPLETE;
+				goto out;
+			}
+
+			if (*start == ';') {
+				const char *next = start + 1;
+				if (next >= end) {
+					ret = RRR_HTTP_PARSE_INCOMPLETE;
+					goto out;
+				}
+				if (*next != ' ' && *next != '\t' && *next != '\r') {
+					bad_client_missing_space_after_comma = 1;
+				}
+				if (*next == ',') {
+					if (RRR_DEBUGLEVEL_1) {
+						RRR_MSG_0("Warning: Comma found after semicolon in HTTP header, bad implementation\n");
+						rrr_http_util_print_where_message(start, end);
+					}
+					start++;
+				}
+			}
+
+			if (*start == ',') {
+				comma_found_do_duplicate_field = 1;
+				break;
+			}
+		} while (prev_subvalue_count != RRR_LL_COUNT(&field->fields));
+
+		// Allow comma and no subvalues?
+	/*		if (RRR_LL_COUNT(&field->fields) == 0) {
+			RRR_MSG_0("No values found after semicolon while parsing HTTP header field subvalues\n");
+			ret = RRR_HTTP_PARSE_SOFT_ERR;
+			goto out;
+		}*/
+
+		if (field->definition != NULL && field->definition->parse != NULL && field->definition->parse(field) != 0) {
+			RRR_MSG_0("Could not parse field '%s' in __rrr_http_parse_header_field\n", field->name);
+			ret = RRR_HTTP_PARSE_SOFT_ERR;
+			goto out;
+		}
+
+		RRR_LL_APPEND(&fields_tmp, field);
+		field = NULL;
+
+		if (!comma_found_do_duplicate_field) {
 			break;
 		}
-	} while (prev_subvalue_count != RRR_LL_COUNT(&field->fields));
-
-	// Allow comma and no subvalues?
-/*		if (RRR_LL_COUNT(&field->fields) == 0) {
-		RRR_MSG_0("No values found after semicolon while parsing HTTP header field subvalues\n");
-		ret = RRR_HTTP_PARSE_SOFT_ERR;
-		goto out;
-	}*/
-
-	if (field->definition != NULL && field->definition->parse != NULL && field->definition->parse(field) != 0) {
-		RRR_MSG_0("Could not parse field '%s' in __rrr_http_parse_header_field\n", field->name);
-		ret = RRR_HTTP_PARSE_SOFT_ERR;
-		goto out;
 	}
 
-	RRR_LL_APPEND(target_list, field);
-
-	if (comma_found_do_duplicate_field) {
-		goto do_duplicate_field;
-	}
-
-	field = NULL;
+	RRR_LL_MERGE_AND_CLEAR_SOURCE_HEAD(target_list, &fields_tmp);
 	*parsed_bytes = (start - start_orig);
 
 	out:
 	if (field != NULL) {
 		__rrr_http_header_field_destroy(field);
 	}
+	RRR_LL_DESTROY(&fields_tmp, struct rrr_http_header_field, __rrr_http_header_field_destroy(node));
 	return ret;
 }
 
@@ -934,7 +970,7 @@ static int __rrr_http_part_parse_chunk_header (
 		size_t start_pos,
 		const char *end
 ) {
-	int ret = RRR_HTTP_PARSE_INCOMPLETE;
+	int ret = RRR_HTTP_PARSE_OK;
 
 	*parsed_bytes = 0;
 	*result_chunk = NULL;
@@ -954,13 +990,15 @@ static int __rrr_http_part_parse_chunk_header (
 	const char *pos = start;
 
 	if (pos >= end) {
-		return RRR_HTTP_PARSE_INCOMPLETE;
+		ret = RRR_HTTP_PARSE_INCOMPLETE;
+		goto out;
 	}
 
 	const char *crlf = rrr_http_util_find_crlf(pos, end);
 
 	if (pos >= end) {
-		return RRR_HTTP_PARSE_INCOMPLETE;
+		ret = RRR_HTTP_PARSE_INCOMPLETE;
+		goto out;
 	}
 
 	// Allow extra \r\n at beginning
@@ -970,49 +1008,52 @@ static int __rrr_http_part_parse_chunk_header (
 //		printf ("Parsed extra CRLF before chunk header\n");
 	}
 
-	if (crlf != NULL) {
-		unsigned long long chunk_length = 0;
+	if (crlf == NULL) {
+		ret = RRR_HTTP_PARSE_INCOMPLETE;
+		goto out;
+	}
 
-		rrr_length parsed_bytes_tmp = 0;
-		if ((ret = rrr_http_util_strtoull(&chunk_length, &parsed_bytes_tmp, pos, crlf, 16)) != 0) {
-			RRR_MSG_0("Error while parsing chunk length, invalid value\n");
-			ret = RRR_HTTP_PARSE_SOFT_ERR;
-			goto out;
-		}
+	unsigned long long chunk_length = 0;
 
-		if (pos + parsed_bytes_tmp == end) {
-			// Chunk header incomplete
-			ret = RRR_HTTP_PARSE_INCOMPLETE;
-			goto out;
-		}
-		else if (ret != 0 || (size_t) crlf - (size_t) pos != parsed_bytes_tmp) {
-			RRR_MSG_0("Error while parsing chunk length, invalid value\n");
-			ret = RRR_HTTP_PARSE_SOFT_ERR;
-			goto out;
-		}
+	rrr_length parsed_bytes_tmp = 0;
+	if ((ret = rrr_http_util_strtoull(&chunk_length, &parsed_bytes_tmp, pos, crlf, 16)) != 0) {
+		RRR_MSG_0("Error while parsing chunk length, invalid value\n");
+		ret = RRR_HTTP_PARSE_SOFT_ERR;
+		goto out;
+	}
 
-		pos += parsed_bytes_tmp;
-		pos += 2; // Plus CRLF after chunk header
+	if (pos + parsed_bytes_tmp == end) {
+		// Chunk header incomplete
+		ret = RRR_HTTP_PARSE_INCOMPLETE;
+		goto out;
+	}
+	else if (ret != 0 || (size_t) crlf - (size_t) pos != parsed_bytes_tmp) {
+		RRR_MSG_0("Error while parsing chunk length, invalid value\n");
+		ret = RRR_HTTP_PARSE_SOFT_ERR;
+		goto out;
+	}
 
-		if (pos + 1 >= end) {
-			ret = RRR_HTTP_PARSE_INCOMPLETE;
-			goto out;
-		}
+	pos += parsed_bytes_tmp;
+	pos += 2; // Plus CRLF after chunk header
 
-		struct rrr_http_chunk *new_chunk = NULL;
-		rrr_length chunk_start = pos - buf;
+	if (pos + 1 >= end) {
+		ret = RRR_HTTP_PARSE_INCOMPLETE;
+		goto out;
+	}
+
+	struct rrr_http_chunk *new_chunk = NULL;
+	rrr_length chunk_start = pos - buf;
 
 //		printf ("First character in chunk: %i\n", *(buf + chunk_start));
 
-		if ((new_chunk = __rrr_http_part_chunk_new(chunk_start, chunk_length)) == NULL) {
-			ret = RRR_HTTP_PARSE_HARD_ERR;
-			goto out;
-		}
-
-		*parsed_bytes = pos - start;
-		*result_chunk = new_chunk;
+	if ((new_chunk = __rrr_http_part_chunk_new(chunk_start, chunk_length)) == NULL) {
+		ret = RRR_HTTP_PARSE_HARD_ERR;
+		goto out;
 	}
 
+	*parsed_bytes = pos - start;
+	*result_chunk = new_chunk;
+
 	out:
 	return ret;
 }
@@ -1044,7 +1085,7 @@ static int __rrr_http_part_parse_header_fields (
 		}
 		else if (crlf == pos) {
 			// Header complete
-			pos += 2;
+			// pos += 2; -- Enable if needed
 			parsed_bytes_total += 2;
 			break;
 		}
@@ -1097,15 +1138,14 @@ static int __rrr_http_part_parse_chunk (
 	}
 
 	struct rrr_http_chunk *new_chunk = NULL;
-	ret = __rrr_http_part_parse_chunk_header (
+
+	if ((ret = __rrr_http_part_parse_chunk_header (
 			&new_chunk,
 			&parsed_bytes_total,
 			buf,
 			start_pos + parsed_bytes_previous_chunk,
 			end
-	);
-
-	if (ret == 0) {
+	)) == 0 && new_chunk != NULL) { // != NULL check due to false positive warning about use of NULL from scan-build
 		RRR_DBG_3("Found new HTTP chunk start %li length %li\n", new_chunk->start, new_chunk->length);
 		RRR_LL_APPEND(chunks, new_chunk);
 
@@ -1191,6 +1231,17 @@ int rrr_http_part_header_fields_iterate (
 	return ret;
 }
 
+void rrr_http_part_header_field_remove (
+		struct rrr_http_part *part,
+		const char *field
+) {
+	RRR_LL_ITERATE_BEGIN(&part->headers, struct rrr_http_header_field);
+		if (strcasecmp(field, node->name) == 0) {
+			RRR_LL_ITERATE_SET_DESTROY();
+		}
+	RRR_LL_ITERATE_END_CHECK_DESTROY(&part->headers, 0; __rrr_http_header_field_destroy(node));
+}
+
 int rrr_http_part_chunks_iterate (
 		struct rrr_http_part *part,
 		const char *data_ptr,
@@ -1321,9 +1372,9 @@ int rrr_http_part_parse (
 
 		part->header_complete = 1;
 
-		const struct rrr_http_header_field *content_type = rrr_http_part_get_header_field(part, "content-type");
-		const struct rrr_http_header_field *content_length = rrr_http_part_get_header_field(part, "content-length");
-		const struct rrr_http_header_field *transfer_encoding = rrr_http_part_get_header_field(part, "transfer-encoding");
+		const struct rrr_http_header_field *content_type = rrr_http_part_header_field_get(part, "content-type");
+		const struct rrr_http_header_field *content_length = rrr_http_part_header_field_get(part, "content-length");
+		const struct rrr_http_header_field *transfer_encoding = rrr_http_part_header_field_get(part, "transfer-encoding");
 
 		if (parse_type == RRR_HTTP_PARSE_REQUEST) {
 			RRR_DBG_3("HTTP request header parse complete\n");
@@ -1343,45 +1394,58 @@ int rrr_http_part_parse (
 				part->request_method = RRR_HTTP_METHOD_OPTIONS;
 			}
 			else if (rrr_posix_strcasecmp(part->request_method_str, "POST") == 0) {
-				if (content_type == NULL || rrr_posix_strcasecmp(content_type->name, "application/octet-stream")) {
-					part->request_method = RRR_HTTP_METHOD_POST_APPLICATION_OCTET_STREAM;
-				}
-				else if (rrr_posix_strcasecmp(content_type->name, "multipart/form-data")) {
-					part->request_method = RRR_HTTP_METHOD_POST_MULTIPART_FORM_DATA;
-				}
-				else if (rrr_posix_strcasecmp(content_type->name, "application/x-www-form-urlencoded")) {
-					part->request_method = RRR_HTTP_METHOD_POST_URLENCODED;
-				}
-				else if (rrr_posix_strcasecmp(content_type->name, "text/plain")) {
-					part->request_method = RRR_HTTP_METHOD_POST_TEXT_PLAIN;
-				}
-				else {
-					RRR_MSG_0("Unknown content-type '%s' in HTTP request\n", content_type->value);
-					ret = RRR_HTTP_PARSE_SOFT_ERR;
-					goto out;
+				part->request_method = RRR_HTTP_METHOD_POST_APPLICATION_OCTET_STREAM;
+
+				if (content_type != NULL) {
+					if (rrr_posix_strcasecmp(content_type->name, "multipart/form-data")) {
+						part->request_method = RRR_HTTP_METHOD_POST_MULTIPART_FORM_DATA;
+					}
+					else if (rrr_posix_strcasecmp(content_type->name, "application/x-www-form-urlencoded")) {
+						part->request_method = RRR_HTTP_METHOD_POST_URLENCODED;
+					}
+					else if (rrr_posix_strcasecmp(content_type->name, "text/plain")) {
+						part->request_method = RRR_HTTP_METHOD_POST_TEXT_PLAIN;
+					}
 				}
 			}
+			else if (rrr_posix_strcasecmp(part->request_method_str, "PUT") == 0) {
+				part->request_method = RRR_HTTP_METHOD_PUT;
+			}
+			else if (rrr_posix_strcasecmp(part->request_method_str, "HEAD") == 0) {
+				part->request_method = RRR_HTTP_METHOD_HEAD;
+			}
+			else if (rrr_posix_strcasecmp(part->request_method_str, "DELETE") == 0) {
+				part->request_method = RRR_HTTP_METHOD_DELETE;
+			}
 			else {
 				RRR_MSG_0("Unknown request method '%s' in HTTP request\n", part->request_method_str);
+				part->response_code = RRR_HTTP_RESPONSE_CODE_ERROR_BAD_REQUEST;
 				ret = RRR_HTTP_PARSE_SOFT_ERR;
 				goto out;
 			}
 
-			if (part->request_method == RRR_HTTP_METHOD_GET || part->request_method == RRR_HTTP_METHOD_OPTIONS) {
+			if (part->request_method == RRR_HTTP_METHOD_GET ||
+				part->request_method == RRR_HTTP_METHOD_OPTIONS ||
+				part->request_method == RRR_HTTP_METHOD_HEAD ||
+				part->request_method == RRR_HTTP_METHOD_DELETE
+			) {
 				if (content_length != NULL && content_length->value_unsigned != 0) {
-					RRR_MSG_0("Content-Length was non-zero for GET or OPTIONS request, this is an error.\n");
+					RRR_MSG_0("Content-Length was non-zero for GET, HEAD, DELETE or OPTIONS request, this is an error.\n");
+					part->response_code = RRR_HTTP_RESPONSE_CODE_ERROR_BAD_REQUEST;
 					ret = RRR_HTTP_PARSE_SOFT_ERR;
 					goto out;
 				}
 
 				if (transfer_encoding != NULL) {
-					RRR_MSG_0("Transfer-Encoding header was set for GET or OPTIONS request, this is an error.\n");
+					RRR_MSG_0("Transfer-Encoding header was set for GET, HEAD, DELETE or OPTIONS request, this is an error.\n");
+					part->response_code = RRR_HTTP_RESPONSE_CODE_ERROR_BAD_REQUEST;
 					ret = RRR_HTTP_PARSE_SOFT_ERR;
 					goto out;
 				}
 
 				if (content_type != NULL) {
-					RRR_MSG_0("Content-Type was set for GET or OPTIONS request, this is an error.\n");
+					RRR_MSG_0("Content-Type was set for GET, HEAD, DELETE or OPTIONS request, this is an error.\n");
+					part->response_code = RRR_HTTP_RESPONSE_CODE_ERROR_BAD_REQUEST;
 					ret = RRR_HTTP_PARSE_SOFT_ERR;
 					goto out;
 				}
@@ -1405,10 +1469,10 @@ int rrr_http_part_parse (
 		else if (transfer_encoding != NULL && rrr_posix_strcasecmp(transfer_encoding->value, "chunked") == 0) {
 			if (parse_type == RRR_HTTP_PARSE_MULTIPART) {
 				RRR_MSG_0("Chunked transfer encoding found in HTTP multipart body, this is not allowed\n");
+				part->response_code = RRR_HTTP_RESPONSE_CODE_ERROR_BAD_REQUEST;
 				ret = RRR_HTTP_SOFT_ERROR;
 				goto out;
 			}
-			ret = RRR_HTTP_PARSE_INCOMPLETE;
 			part->is_chunked = 1;
 			RRR_DBG_3("HTTP chunked transfer encoding specified\n");
 			goto parse_chunked;
@@ -1581,7 +1645,7 @@ static int __rrr_http_part_process_multipart_part (
 		boundary_pos -= 2;
 	}
 
-	if ((ret = rrr_http_part_new(&new_part)) != 0) {
+	if (rrr_http_part_new(&new_part) != 0) {
 		RRR_MSG_0("Could not allocate new part in __rrr_http_part_process_multipart_part\n");
 		ret = RRR_HTTP_PARSE_HARD_ERR;
 		goto out;
@@ -1599,12 +1663,12 @@ static int __rrr_http_part_process_multipart_part (
 			boundary_pos,
 			RRR_HTTP_PARSE_MULTIPART
 	)) != 0) {
-		if (ret != RRR_HTTP_PARSE_INCOMPLETE) {
-			RRR_MSG_0("Failed to parse part from HTTP multipart request\n");
+		// Incomplete return is normal, the parser does not know about boundaries
+		ret &= ~(RRR_HTTP_PARSE_INCOMPLETE);
+		if (ret != 0) {
+			RRR_MSG_0("Failed to parse part from HTTP multipart request return was %i\n", ret);
 			goto out;
 		}
-		// Incomplete return is normal, the parser does not know about boundaries
-		ret = RRR_HTTP_PARSE_OK;
 	}
 
 	if (new_part->headroom_length != 0) {
@@ -1658,7 +1722,7 @@ int rrr_http_part_process_multipart (
 		goto out;
 	}
 
-	const struct rrr_http_field *boundary = __rrr_http_part_get_header_field_subvalue(part, "content-type", "boundary");
+	const struct rrr_http_field *boundary = __rrr_http_part_header_field_subvalue_get(part, "content-type", "boundary");
 	if (boundary == NULL || boundary->value == NULL || *(boundary->value) == '\0') {
 		RRR_MSG_0("No multipart boundary found in content-type of HTTP header\n");
 		ret = RRR_HTTP_PARSE_SOFT_ERR;
@@ -1926,7 +1990,7 @@ int rrr_http_part_extract_post_and_query_fields (
 		RRR_LL_ITERATE_BEGIN(target, struct rrr_http_part);
 			RRR_HTTP_PART_DECLARE_DATA_START_AND_END(node, data_ptr);
 
-			const struct rrr_http_field *field_name = __rrr_http_part_get_header_field_subvalue(node, "content-disposition", "name");
+			const struct rrr_http_field *field_name = __rrr_http_part_header_field_subvalue_get(node, "content-disposition", "name");
 			if (field_name == NULL || field_name->value == NULL || field_name->value_size == 0) {
 				RRR_DBG_1("Warning: Unknown field or invalid content-disposition of multipart part\n");
 				RRR_LL_ITERATE_NEXT();
@@ -1942,7 +2006,7 @@ int rrr_http_part_extract_post_and_query_fields (
 				goto out;
 			}
 
-			const struct rrr_http_header_field *field_content_type = rrr_http_part_get_header_field(node, "content-type");
+			const struct rrr_http_header_field *field_content_type = rrr_http_part_header_field_get(node, "content-type");
 
 			if (field_content_type != NULL && field_content_type->value != NULL) {
 				if ((ret = rrr_http_field_set_content_type(
@@ -1984,7 +2048,7 @@ int rrr_http_part_merge_chunks (
 	*result_data = NULL;
 
 	char *data_new = NULL;
-	const size_t top_length = part->headroom_length + part->header_length;
+	const size_t top_length = RRR_HTTP_PART_TOP_LENGTH(part);
 	size_t new_buf_size = 0;
 
 	new_buf_size += top_length;
@@ -2014,10 +2078,10 @@ int rrr_http_part_merge_chunks (
 	part->data_length = wpos;
 
 	*result_data = data_new;
-	data_new = NULL;
 
+	// goto out; out_free:
+	// RRR_FREE_IF_NOT_NULL(data_new); -- Enable if needed
 	out:
-	RRR_FREE_IF_NOT_NULL(data_new);
 	return ret;
 }
 
