@@ -31,15 +31,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "http_util.h"
 #include "http_part.h"
 
-#include "../posix.h"
-#include "../gnu.h"
-#include "../base64.h"
-#include "../linked_list.h"
 #include "../net_transport/net_transport.h"
 #include "../random.h"
 #include "../read.h"
-#include "../rrr_time.h"
-#include "../macro_utils.h"
+#include "../string_builder.h"
+#include "../util/posix.h"
+#include "../util/gnu.h"
+#include "../util/base64.h"
+#include "../util/linked_list.h"
+#include "../util/rrr_time.h"
+#include "../util/macro_utils.h"
 
 static void __rrr_http_session_destroy (struct rrr_http_session *session) {
 	RRR_FREE_IF_NOT_NULL(session->uri_str);
@@ -158,7 +159,7 @@ int rrr_http_session_transport_ctx_set_endpoint (
 	return 0;
 }
 
-int rrr_http_session_transport_ctx_client_new (
+int rrr_http_session_transport_ctx_client_new_or_clean (
 		struct rrr_net_transport_handle *handle,
 		enum rrr_http_method method,
 		const char *user_agent
@@ -167,29 +168,42 @@ int rrr_http_session_transport_ctx_client_new (
 
 	struct rrr_http_session *session = NULL;
 
-	if ((__rrr_http_session_allocate(&session)) != 0) {
-		RRR_MSG_0("Could not allocate memory in rrr_http_session_transport_ctx_client_new\n");
-		ret = 1;
-		goto out;
-	}
-
-	session->method = method;
-	session->is_client = 1;
-	session->uri_str = strdup("/");
-
-	if (session->uri_str == NULL) {
-		RRR_MSG_0("Could not allocate memory in rrr_http_session_new B\n");
-		ret = 1;
-		goto out;
-	}
-
-	if (user_agent != NULL && *user_agent != '\0') {
-		session->user_agent = strdup(user_agent);
-		if (session->user_agent == NULL) {
-			RRR_MSG_0("Could not allocate memory in rrr_http_session_new D\n");
+	// With keepalive connections, structures are already present in transport handle
+	if (!rrr_net_transport_ctx_handle_has_application_data(handle)) {
+		if ((__rrr_http_session_allocate(&session)) != 0) {
+			RRR_MSG_0("Could not allocate memory in rrr_http_session_transport_ctx_client_new\n");
 			ret = 1;
 			goto out;
 		}
+
+		session->method = method;
+		session->is_client = 1;
+		session->uri_str = strdup("/");
+
+		if (session->uri_str == NULL) {
+			RRR_MSG_0("Could not allocate memory in rrr_http_session_new B\n");
+			ret = 1;
+			goto out;
+		}
+
+		if (user_agent != NULL && *user_agent != '\0') {
+			session->user_agent = strdup(user_agent);
+			if (session->user_agent == NULL) {
+				RRR_MSG_0("Could not allocate memory in rrr_http_session_new D\n");
+				ret = 1;
+				goto out;
+			}
+		}
+
+		// Transport framework responsible for cleaning up
+		rrr_net_transport_ctx_handle_application_data_bind (
+				handle,
+				session,
+				__rrr_http_session_destroy_void
+		);
+	}
+	else {
+		session = handle->application_private_ptr;
 	}
 
 	if (__rrr_http_session_prepare_part(&session->request_part) != 0) {
@@ -198,13 +212,6 @@ int rrr_http_session_transport_ctx_client_new (
 		goto out;
 	}
 
-	// Transport framework responsible for cleaning up
-	rrr_net_transport_ctx_handle_application_data_bind (
-			handle,
-			session,
-			__rrr_http_session_destroy_void
-	);
-
 	session = NULL;
 
 	out:
@@ -258,6 +265,30 @@ void rrr_http_session_query_fields_dump (
 	rrr_http_field_collection_dump(&session->request_part->fields);
 }
 
+int rrr_http_session_set_keepalive (
+		struct rrr_http_session *session,
+		int set
+) {
+	int ret = 0;
+
+	if (session->request_part == NULL) {
+		RRR_BUG("BUG: rrr_http_session_set_keepalive called without request part being initialized first\n");
+	}
+
+	rrr_http_part_header_field_remove(session->request_part, "Connection");
+
+	if (set) {
+		ret = rrr_http_part_header_field_push(session->request_part, "Connection", "keep-alive");
+	}
+
+	return ret;
+}
+
+static void __rrr_http_session_free_dbl_ptr (void *ptr) {
+	void *to_free = *((void **) ptr);
+	RRR_FREE_IF_NOT_NULL(to_free);
+}
+
 static int __rrr_http_session_multipart_form_data_body_send_wrap_chunk (
 		struct rrr_net_transport_handle *handle,
 		const void *data,
@@ -312,7 +343,7 @@ static int __rrr_http_session_multipart_field_send (
 		}
 
 		if ((ret = rrr_asprintf (&name_buf_full, "; name=%s", name_buf)) <= 0) {
-			RRR_MSG_0("Could not create name_buf_full in __rrr_http_session_multipart_field_send\n");
+			RRR_MSG_0("Could not create name_buf_full in __rrr_http_session_multipart_field_send return was %i\n", ret);
 			ret = 1;
 			goto out;
 		}
@@ -320,7 +351,7 @@ static int __rrr_http_session_multipart_field_send (
 
 	if (node->content_type != NULL && *(node->content_type) != '\0') {
 		if ((ret = rrr_asprintf (&content_type_buf, "Content-Type: %s\r\n", node->content_type)) <= 0) {
-			RRR_MSG_0("Could not create content_type_buf in __rrr_http_session_multipart_field_send\n");
+			RRR_MSG_0("Could not create content_type_buf in __rrr_http_session_multipart_field_send return was %i\n", ret);
 			ret = 1;
 			goto out;
 		}
@@ -337,7 +368,7 @@ static int __rrr_http_session_multipart_field_send (
 			(name_buf_full != NULL ? name_buf_full : ""),
 			(content_type_buf != NULL ? content_type_buf : "")
 	)) < 0) {
-		RRR_MSG_0("Could not create content type string and body  in __rrr_http_session_multipart_field_send\n");
+		RRR_MSG_0("Could not create content type string and body  in __rrr_http_session_multipart_field_send return was %i\n", ret);
 		ret = 1;
 		goto out;
 	}
@@ -372,10 +403,13 @@ static int __rrr_http_session_multipart_form_data_body_send (
 	char *body_buf = NULL;
 	char *boundary_buf = NULL;
 
+	pthread_cleanup_push(__rrr_http_session_free_dbl_ptr, &body_buf);
+	pthread_cleanup_push(__rrr_http_session_free_dbl_ptr, &boundary_buf);
+
 	// RFC7578
 
 	if ((ret = rrr_asprintf (&boundary_buf, "RRR%u", (unsigned int) rrr_rand())) < 0) {
-		RRR_MSG_0("Could not create boundary_buf string in __rrr_http_session_multipart_form_data_body_send\n");
+		RRR_MSG_0("Could not create boundary_buf string in __rrr_http_session_multipart_form_data_body_send return was %i\n", ret);
 		ret = 1;
 		goto out;
 	}
@@ -388,12 +422,13 @@ static int __rrr_http_session_multipart_form_data_body_send (
 				"Transfer-Encoding: chunked\r\n\r\n",
 				boundary_buf
 		)) < 0) {
-			RRR_MSG_0("Could not create content type string in __rrr_http_session_multipart_form_data_body_send\n");
+			RRR_MSG_0("Could not create content type string in __rrr_http_session_multipart_form_data_body_send return was %i\n", ret);
+			ret = 1;
 			goto out;
 		}
 
 		if ((ret = rrr_net_transport_ctx_send_blocking(handle, body_buf, strlen(body_buf))) != 0) {
-			RRR_MSG_0("Could not send first part of HTTP request in __rrr_http_session_multipart_form_data_body_send\n");
+			RRR_DBG_1("Could not send first part of HTTP request in __rrr_http_session_multipart_form_data_body_send\n");
 			goto out;
 		}
 	}
@@ -415,7 +450,7 @@ static int __rrr_http_session_multipart_form_data_body_send (
 				"\r\n--%s--\r\n",  // <-- ONE CRLF AFTER BODY AND ONE AT THE VERY END
 				boundary_buf
 		)) < 0) {
-			RRR_MSG_0("Could not create last boundary in __rrr_http_session_multipart_form_data_body_send\n");
+			RRR_MSG_0("Could not create last boundary in __rrr_http_session_multipart_form_data_body_send return was %i\n", ret);
 			ret = 1;
 			goto out;
 		}
@@ -427,13 +462,13 @@ static int __rrr_http_session_multipart_form_data_body_send (
 	}
 
 	if ((ret = rrr_net_transport_ctx_send_blocking(handle, "0\r\n\r\n", 5)) != 0) {
-		RRR_MSG_0("Could not send terminating chunk of HTTP request in __rrr_http_session_multipart_form_data_body_send\n");
+		RRR_DBG_1("Could not send terminating chunk of HTTP request in __rrr_http_session_multipart_form_data_body_send\n");
 		goto out;
 	}
 
 	out:
-	RRR_FREE_IF_NOT_NULL(body_buf);
-	RRR_FREE_IF_NOT_NULL(boundary_buf);
+	pthread_cleanup_pop(1);
+	pthread_cleanup_pop(1);
 
 	return ret;
 }
@@ -448,6 +483,9 @@ static int __rrr_http_session_post_x_www_form_body_send (
 	char *body_buf = NULL;
 	char *header_buf = NULL;
 
+	pthread_cleanup_push(__rrr_http_session_free_dbl_ptr, &body_buf);
+	pthread_cleanup_push(__rrr_http_session_free_dbl_ptr, &header_buf);
+
 	rrr_length body_size = 0;
 	if (no_urlencoding == 0) {
 		body_buf = rrr_http_field_collection_to_urlencoded_form_data(&body_size, &session->request_part->fields);
@@ -468,24 +506,24 @@ static int __rrr_http_session_post_x_www_form_body_send (
 			"Content-Length: %" PRIrrrl "\r\n\r\n",
 			body_size
 	)) < 0) {
-		RRR_MSG_0("Could not create content type string in __rrr_http_session_send_get_body\n");
+		RRR_MSG_0("Could not create content type string in __rrr_http_session_send_get_body return was %i\n", ret);
 		ret = 1;
 		goto out;
 	}
 
 	if ((ret = rrr_net_transport_ctx_send_blocking (handle, header_buf, strlen(header_buf))) != 0) {
-		RRR_MSG_0("Could not send GET body header in __rrr_http_session_send_get_body\n");
+		RRR_DBG_1("Could not send GET body header in __rrr_http_session_send_get_body\n");
 		goto out;
 	}
 
 	if ((ret = rrr_net_transport_ctx_send_blocking (handle, body_buf, body_size)) != 0) {
-		RRR_MSG_0("Could not send GET body in __rrr_http_session_send_get_body\n");
+		RRR_DBG_1("Could not send GET body in __rrr_http_session_send_get_body\n");
 		goto out;
 	}
 
 	out:
-	RRR_FREE_IF_NOT_NULL(header_buf);
-	RRR_FREE_IF_NOT_NULL(body_buf);
+	pthread_cleanup_pop(1);
+	pthread_cleanup_pop(1);
 	return ret;
 }
 
@@ -493,6 +531,27 @@ struct rrr_http_session_send_request_callback_data {
 	const char *host;
 };
 
+static int __rrr_http_session_request_send_make_headers_callback (
+		struct rrr_http_header_field *field,
+		void *arg
+) {
+	struct rrr_string_builder *builder = arg;
+
+	// Note : Only plain values supported
+	if (field->value == NULL || *(field->value) == '\0') {
+		return 0;
+	}
+
+	int ret = 0;
+
+	ret |= rrr_string_builder_append(builder, field->name);
+	ret |= rrr_string_builder_append(builder, ": ");
+	ret |= rrr_string_builder_append(builder, field->value);
+	ret |= rrr_string_builder_append(builder, "\r\n");
+
+	return ret;
+}
+
 static int __rrr_http_session_request_send (struct rrr_net_transport_handle *handle, void *arg) {
 	struct rrr_http_session_send_request_callback_data *callback_data = arg;
 	struct rrr_http_session *session = handle->application_private_ptr;
@@ -502,15 +561,30 @@ static int __rrr_http_session_request_send (struct rrr_net_transport_handle *han
 	char *request_buf = NULL;
 	char *host_buf = NULL;
 	char *user_agent_buf = NULL;
+	char *uri_tmp = NULL;
+	char *extra_uri_tmp = NULL;
+
+	pthread_cleanup_push(__rrr_http_session_free_dbl_ptr, &request_buf);
+	pthread_cleanup_push(__rrr_http_session_free_dbl_ptr, &host_buf);
+	pthread_cleanup_push(__rrr_http_session_free_dbl_ptr, &user_agent_buf);
+	pthread_cleanup_push(__rrr_http_session_free_dbl_ptr, &uri_tmp);
+	pthread_cleanup_push(__rrr_http_session_free_dbl_ptr, &extra_uri_tmp);
 
 	rrr_length extra_uri_size = 0;
-	char *extra_uri_tmp = NULL;
 	const char *extra_uri_separator = "";
 
-	char *uri_tmp = NULL;
-
 	const char *uri_to_use = session->uri_str;
 
+	struct rrr_string_builder *header_builder = NULL;
+
+	if (rrr_string_builder_new(&header_builder) != 0) {
+		RRR_MSG_0("Failed to create string builder in __rrr_http_session_request_send\n");
+		ret = 1;
+		goto out_final;
+	}
+
+	pthread_cleanup_push(rrr_string_builder_destroy_void, header_builder);
+
 	host_buf = rrr_http_util_quote_header_value(callback_data->host, '"', '"');
 	if (host_buf == NULL) {
 		RRR_MSG_0("Invalid host '%s' in rrr_http_session_send_request\n", callback_data->host);
@@ -572,16 +646,38 @@ static int __rrr_http_session_request_send (struct rrr_net_transport_handle *han
 			host_buf,
 			user_agent_buf
 	)) < 0) {
-		RRR_MSG_0("Error while making request string in rrr_http_session_send_request\n");
+		RRR_MSG_0("Error while making request string in rrr_http_session_send_request return was %i\n", ret);
 		ret = 1;
 		goto out;
 	}
 
 	if ((ret = rrr_net_transport_ctx_send_blocking (handle, request_buf, strlen(request_buf))) != 0) {
-		RRR_MSG_0("Could not send first part of HTTP request in rrr_http_session_send_request\n");
+		RRR_DBG_1("Could not send first part of HTTP request header in rrr_http_session_send_request\n");
 		goto out;
 	}
 
+	rrr_string_builder_clear(header_builder);
+
+	if (rrr_http_part_header_fields_iterate (
+			session->request_part,
+			__rrr_http_session_request_send_make_headers_callback,
+			header_builder
+	) != 0) {
+		RRR_MSG_0("Failed to make header fields in rrr_http_session_send_request\n");
+		ret = 1;
+		goto out;
+	}
+
+	ssize_t header_builder_length = rrr_string_builder_length(header_builder);
+	if (header_builder_length > 0) {
+		RRR_FREE_IF_NOT_NULL(request_buf);
+		request_buf = rrr_string_builder_buffer_takeover(header_builder);
+		if ((ret = rrr_net_transport_ctx_send_blocking (handle, request_buf, header_builder_length)) != 0) {
+			RRR_MSG_0("Could not send second part of HTTP request header in rrr_http_session_send_request\n");
+			goto out;
+		}
+	}
+
 	if (session->method != RRR_HTTP_METHOD_GET && RRR_LL_COUNT(&session->request_part->fields) > 0) {
 		if (session->method == RRR_HTTP_METHOD_POST_MULTIPART_FORM_DATA) {
 			if ((ret = __rrr_http_session_multipart_form_data_body_send (handle)) != 0) {
@@ -617,12 +713,14 @@ static int __rrr_http_session_request_send (struct rrr_net_transport_handle *han
 	}
 
 	out:
-	RRR_FREE_IF_NOT_NULL(user_agent_buf);
-	RRR_FREE_IF_NOT_NULL(host_buf);
-	RRR_FREE_IF_NOT_NULL(request_buf);
-	RRR_FREE_IF_NOT_NULL(extra_uri_tmp);
-	RRR_FREE_IF_NOT_NULL(uri_tmp);
-	return ret;
+		pthread_cleanup_pop(1);
+	out_final:
+		pthread_cleanup_pop(1);
+		pthread_cleanup_pop(1);
+		pthread_cleanup_pop(1);
+		pthread_cleanup_pop(1);
+		pthread_cleanup_pop(1);
+		return ret;
 }
 
 int rrr_http_session_transport_ctx_request_send (
@@ -636,12 +734,26 @@ int rrr_http_session_transport_ctx_request_send (
 	return __rrr_http_session_request_send(handle, &callback_data);
 }
 
+int rrr_http_session_transport_ctx_raw_request_send (
+		struct rrr_net_transport_handle *handle,
+		const char *raw_request_data,
+		size_t raw_request_size
+) {
+	if (raw_request_size == 0) {
+		RRR_BUG("BUG: Received 0 size in rrr_http_session_transport_ctx_raw_request_send\n");
+	}
+	return rrr_net_transport_ctx_send_blocking (handle, raw_request_data, raw_request_size);
+}
+
 struct rrr_http_session_receive_data {
 	struct rrr_net_transport_handle *handle;
 	ssize_t parse_complete_pos;
 	ssize_t received_bytes; // Used only for stall timeout
+	rrr_http_unique_id unique_id;
 	int (*callback)(RRR_HTTP_SESSION_RECEIVE_CALLBACK_ARGS);
 	void *callback_arg;
+	int (*raw_callback)(RRR_HTTP_SESSION_RAW_RECEIVE_CALLBACK_ARGS);
+	void *raw_callback_arg;
 };
 
 static int __rrr_http_session_response_receive_callback (
@@ -651,7 +763,7 @@ static int __rrr_http_session_response_receive_callback (
 	struct rrr_http_session_receive_data *receive_data = arg;
 	struct rrr_http_session *session = receive_data->handle->application_private_ptr;
 
-	(void)(read_session);
+	int ret = 0;
 
 	if (RRR_DEBUGLEVEL_3) {
 		rrr_http_part_dump_header(session->response_part);
@@ -663,17 +775,34 @@ static int __rrr_http_session_response_receive_callback (
 			session->response_part->header_length
 	);
 
-	int ret = receive_data->callback (
+	if (receive_data->raw_callback != NULL) {
+		if ((ret = receive_data->raw_callback (
+				read_session->rx_buf_ptr,
+				read_session->rx_buf_wpos,
+				0,
+				receive_data->raw_callback_arg
+		)) != 0) {
+			RRR_MSG_0("Error %i from raw callback in __rrr_http_session_response_receive_callback\n", ret);
+			goto out;
+		}
+	}
+
+	if ((ret = receive_data->callback (
+			receive_data->handle,
 			session->request_part,
 			session->response_part,
 			read_session->rx_buf_ptr,
 			(const struct sockaddr *) &read_session->src_addr,
 			read_session->src_addr_len,
 			read_session->rx_overshoot_size,
+			0,
 			receive_data->callback_arg
-	);
+	)) != 0) {
+		goto out;
+	}
 
 	// ALWAYS destroy parts
+	out:
 	__rrr_http_session_destroy_part(&session->response_part);
 	__rrr_http_session_destroy_part(&session->request_part);
 
@@ -699,6 +828,8 @@ static int __rrr_http_session_send_header_field_callback (struct rrr_http_header
 		RRR_BUG("BUG: Subvalues were present in __rrr_http_session_send_header_field_callback, this is not supported\n");
 	}
 
+	pthread_cleanup_push(__rrr_http_session_free_dbl_ptr, &send_data);
+
 	if ((send_data_length = rrr_asprintf(&send_data, "%s: %s\r\n", field->name, field->value)) <= 0) {
 		RRR_MSG_0("Could not allocate memory for header line in __rrr_http_session_send_header_field_callback\n");
 		ret = 1;
@@ -722,12 +853,12 @@ static int __rrr_http_session_send_header_field_callback (struct rrr_http_header
 	}
 
 	if ((ret = rrr_net_transport_ctx_send_blocking(callback_data->handle, send_data, send_data_length)) != 0) {
-		RRR_MSG_0("Error: Send failed in __rrr_http_session_send_header_field_callback\n");
+		RRR_DBG_1("Error: Send failed in __rrr_http_session_send_header_field_callback\n");
 		goto out;
 	}
 
 	out:
-	RRR_FREE_IF_NOT_NULL(send_data);
+	pthread_cleanup_pop(1);
 	return ret;
 }
 
@@ -742,6 +873,18 @@ static int __rrr_http_session_transport_ctx_send_response (
 	if (response_part == NULL) {
 		RRR_BUG("BUG: Response part was NULL in rrr_http_session_send_response\n");
 	}
+
+	if (response_part->response_raw_data != NULL) {
+		if ((ret = rrr_net_transport_ctx_send_blocking (
+				handle,
+				response_part->response_raw_data,
+				response_part->response_raw_data_size
+		)) != 0 ) {
+			goto out_err;
+		}
+		goto out;
+	}
+
 	if (response_part->response_code == 0) {
 		RRR_BUG("BUG: Response code was not set in rrr_http_session_send_response\n");
 	}
@@ -764,6 +907,12 @@ static int __rrr_http_session_transport_ctx_send_response (
 		case RRR_HTTP_RESPONSE_CODE_INTERNAL_SERVER_ERROR:
 			response_str = "HTTP/1.1 500 Internal Server Error\r\n";
 			break;
+		case RRR_HTTP_RESPONSE_CODE_GATEWAY_TIMEOUT:
+			response_str = "HTTP/1.1 504 Gateway Timeout\r\n";
+			break;
+		case RRR_HTTP_RESPONSE_CODE_VERSION_NOT_SUPPORTED:
+			response_str = "HTTP/1.1 504 Version Not Supported\r\n";
+			break;
 		default:
 			RRR_BUG("BUG: Response code %i not implemented in rrr_http_session_send_response\n",
 					response_part->response_code);
@@ -816,6 +965,18 @@ static int __rrr_http_session_request_receive_callback (
 			session->request_part->header_length
 	);
 
+	if (receive_data->raw_callback != NULL) {
+		if ((ret = receive_data->raw_callback (
+				read_session->rx_buf_ptr,
+				read_session->rx_buf_wpos,
+				receive_data->unique_id,
+				receive_data->raw_callback_arg
+		)) != 0) {
+			RRR_MSG_0("Error %i from raw callback in __rrr_http_session_request_receive_callback\n", ret);
+			goto out;
+		}
+	}
+
 	if ((ret = rrr_http_part_merge_chunks(&merged_chunks, session->request_part, read_session->rx_buf_ptr)) != 0) {
 		goto out;
 	}
@@ -840,12 +1001,14 @@ static int __rrr_http_session_request_receive_callback (
 	}
 
 	if ((ret = receive_data->callback (
+			receive_data->handle,
 			session->request_part,
 			session->response_part,
 			data_to_use,
 			(const struct sockaddr *) &read_session->src_addr,
 			read_session->src_addr_len,
 			read_session->rx_overshoot_size,
+			receive_data->unique_id,
 			receive_data->callback_arg
 	)) != RRR_HTTP_OK) {
 		goto out;
@@ -921,7 +1084,7 @@ static int __rrr_http_session_receive_get_target_size (
 	} while (parsed_bytes != 0 && ret == RRR_HTTP_PARSE_INCOMPLETE);
 
 	if (target_size > SSIZE_MAX) {
-		RRR_MSG_0("Target size %llu exceeds maximum value of %lld while parsing HTTP part\n",
+		RRR_MSG_0("Target size %lu exceeds maximum value of %li while parsing HTTP part\n",
 				target_size, SSIZE_MAX);
 		ret = RRR_NET_TRANSPORT_READ_SOFT_ERROR;
 		goto out;
@@ -952,8 +1115,11 @@ int rrr_http_session_transport_ctx_receive (
 		uint64_t timeout_stall_us,
 		uint64_t timeout_total_us,
 		ssize_t read_max_size,
+		rrr_http_unique_id unique_id,
 		int (*callback)(RRR_HTTP_SESSION_RECEIVE_CALLBACK_ARGS),
-		void *callback_arg
+		void *callback_arg,
+		int (*raw_callback)(RRR_HTTP_SESSION_RAW_RECEIVE_CALLBACK_ARGS),
+		void *raw_callback_arg
 ) {
 	struct rrr_http_session *session = handle->application_private_ptr;
 
@@ -963,8 +1129,11 @@ int rrr_http_session_transport_ctx_receive (
 			handle,
 			0,
 			0,
+			unique_id,
 			callback,
-			callback_arg
+			callback_arg,
+			raw_callback,
+			raw_callback_arg
 	};
 
 	// Parts are prepared when a new client is created and /after/
@@ -988,7 +1157,6 @@ int rrr_http_session_transport_ctx_receive (
 					4096,
 					65535,
 					read_max_size,
-					0, // Flags
 					__rrr_http_session_receive_get_target_size,
 					&callback_data,
 					session->is_client
