@@ -24,40 +24,48 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "modules.h"
 #include "configuration.h"
-#include "instance_collection.h"
+#include "instance_friends.h"
 #include "threads.h"
+#include "poll_helper.h"
+#include "util/linked_list.h"
 
 struct rrr_stats_instance;
 struct rrr_cmodule;
-struct rrr_poll_collection;
 struct rrr_fork_handler;
 struct rrr_stats_engine;
 struct rrr_message_broker;
 typedef void rrr_message_broker_costumer_handle;
 struct rrr_mqtt_topic_token;
 
-struct rrr_instance_metadata {
-	struct rrr_instance_metadata *next;
-	struct rrr_instance_dynamic_data *dynamic_data;
-	struct rrr_instance_thread_data *thread_data;
-	struct rrr_instance_collection senders;
-	struct rrr_instance_collection wait_for;
-	struct rrr_instance_config *config;
+struct rrr_instance {
+	RRR_LL_NODE(struct rrr_instance);
+	// Managed by this struct
+	struct rrr_instance_module_data *module_data;
+	struct rrr_instance_friend_collection senders;
+	struct rrr_instance_friend_collection wait_for;
 	struct rrr_signal_handler *signal_handler;
-	unsigned long int senders_count;
 	char *topic_filter;
 	struct rrr_mqtt_topic_token *topic_first_token;
+
+	// Static members
+	unsigned long int senders_count;
+
+	// Shortcuts
+	struct rrr_instance_config_data *config;
+	struct rrr_thread *thread;
 };
 
-#define INSTANCE_M_NAME(instance) instance->dynamic_data->instance_name
-#define INSTANCE_M_MODULE_NAME(instance) instance->dynamic_data->module_name
+#define INSTANCE_M_THREAD(instance) instance->thread
+#define INSTANCE_M_NAME(instance) instance->module_data->instance_name
+#define INSTANCE_M_MODULE_TYPE(instance) instance->module_data->type
+#define INSTANCE_M_MODULE_NAME(instance) instance->module_data->module_name
 
-struct rrr_instance_metadata_collection {
-	int length;
-	struct rrr_instance_metadata *first_entry;
+struct rrr_instance_collection {
+	RRR_LL_HEAD(struct rrr_instance);
+	struct rrr_instance *first_entry;
 };
 
-struct rrr_instance_dynamic_data {
+struct rrr_instance_module_data {
 	const char *instance_name;
 	const char *module_name;
 	unsigned int type;
@@ -67,32 +75,33 @@ struct rrr_instance_dynamic_data {
 	void *dl_ptr;
 	void *private_data;
 	void (*unload)(void);
-	int (*signal_handler)(int s, void *priv);
-	struct rrr_instance_metadata_collection *all_instances;
+	struct rrr_instance_collection *all_instances;
 };
 
 #define INSTANCE_D_NAME(thread_data) thread_data->init_data.module->instance_name
 #define INSTANCE_D_MODULE_NAME(thread_data) thread_data->init_data.module->module_name
+#define INSTANCE_D_THREAD(thread_data) thread_data->thread
+#define INSTANCE_D_INSTANCE(thread_data) thread_data->init_data.instance
 
-struct rrr_instance_thread_init_data {
+struct rrr_instance_runtime_init_data {
 	struct cmd_data *cmd_data;
-	struct rrr_instance_config *instance_config;
+
+	// Shortcuts to other structures
+	struct rrr_instance_config_data *instance_config;
 	struct rrr_config *global_config;
-	struct rrr_instance_dynamic_data *module;
-	struct rrr_instance_collection *senders;
+	struct rrr_instance_module_data *module;
+	struct rrr_instance_friend_collection *senders;
 	struct rrr_stats_engine *stats;
 	struct rrr_message_broker *message_broker;
 	struct rrr_fork_handler *fork_handler;
 	const struct rrr_mqtt_topic_token *topic_first_token;
 	const char *topic_str;
+	struct rrr_instance *instance;
 };
 
-struct rrr_instance_thread_data {
-	struct rrr_instance_thread_init_data init_data;
+struct rrr_instance_runtime_data {
+	struct rrr_instance_runtime_init_data init_data;
 
-	int used_by_ghost;
-
-	struct rrr_thread *thread;
 	rrr_message_broker_costumer_handle *message_broker_handle;
 
 	void *private_data;
@@ -100,11 +109,14 @@ struct rrr_instance_thread_data {
 	char private_memory[RRR_MODULE_PRIVATE_MEMORY_SIZE];
 	char preload_memory[RRR_MODULE_PRELOAD_MEMORY_SIZE];
 
+	// Shorcut which is set by thread itself after starting
+	struct rrr_thread *thread;
+
 	// Not used by all modules but managed by instances framework.
 	// * Init and cleanup in intermediate thread entry function
 	// * Cleanup in ghost handler
 	struct rrr_cmodule *cmodule;
-	struct rrr_poll_collection *poll;
+	struct rrr_poll_collection poll;
 	struct rrr_stats_instance *stats;
 };
 
@@ -121,72 +133,56 @@ struct rrr_instance_thread_data {
 #define INSTANCE_D_BROKER_ARGS(thread_data) \
 		thread_data->init_data.message_broker, thread_data->message_broker_handle
 
-#define RRR_INSTANCE_LOOP(target,collection) \
-	for (struct rrr_instance_metadata *target = collection->first_entry; target != NULL; target = target->next)
-
-struct rrr_instance_metadata *rrr_instance_find_by_thread (
-		struct rrr_instance_metadata_collection *collection,
+struct rrr_instance *rrr_instance_find_by_thread (
+		struct rrr_instance_collection *instances,
 		struct rrr_thread *thread
 );
 int rrr_instance_check_threads_stopped(
-		struct rrr_instance_metadata_collection *target
-);
-void rrr_instance_free_all_thread_data(
-		struct rrr_instance_metadata_collection *target
+		struct rrr_instance_collection *target
 );
 int rrr_instance_count_library_users (
-		struct rrr_instance_metadata_collection *target,
+		struct rrr_instance_collection *target,
 		void *dl_ptr
 );
 void rrr_instance_unload_all(
-		struct rrr_instance_metadata_collection *target
+		struct rrr_instance_collection *target
 );
-void rrr_instance_metadata_collection_destroy (
-		struct rrr_instance_metadata_collection *target
+void rrr_instance_collection_clear (
+		struct rrr_instance_collection *target
 );
-int rrr_instance_metadata_collection_new (
-		struct rrr_instance_metadata_collection **target
+int rrr_instance_collection_new (
+		struct rrr_instance_collection **target
 );
 int rrr_instance_load_and_save (
-		struct rrr_instance_metadata_collection *instances,
-		struct rrr_instance_config *instance_config,
+		struct rrr_instance_collection *instances,
+		struct rrr_instance_config_data *instance_config,
 		const char **library_paths
 );
-struct rrr_instance_metadata *rrr_instance_find (
-		struct rrr_instance_metadata_collection *target,
+struct rrr_instance *rrr_instance_find (
+		struct rrr_instance_collection *target,
 		const char *name
 );
-unsigned int rrr_instance_metadata_collection_count (
-		struct rrr_instance_metadata_collection *collection
+unsigned int rrr_instance_collection_count (
+		struct rrr_instance_collection *collection
 );
-void rrr_instance_destroy_thread(
-		struct rrr_instance_thread_data *data
+struct rrr_instance_runtime_data *rrr_instance_runtime_data_new (
+		struct rrr_instance_runtime_init_data *init_data
 );
-void rrr_instance_destroy_thread_by_ghost (
-		void *private_data
+void *rrr_instance_thread_entry_intermediate (
+		struct rrr_thread *thread
 );
-struct rrr_instance_thread_data *rrr_instance_new_thread(
-		struct rrr_instance_thread_init_data *init_data
-);
-int rrr_instance_preload_thread(
-		struct rrr_thread_collection *collection,
-		struct rrr_instance_thread_data *data
-);
-int rrr_instance_start_thread (
-		struct rrr_instance_thread_data *data
-);
-int rrr_instance_process_from_config(
-		struct rrr_instance_metadata_collection *instances,
+int rrr_instance_create_from_config (
+		struct rrr_instance_collection *instances,
 		struct rrr_config *config,
 		const char **library_paths
 );
 int rrr_instance_count_receivers_of_self (
-		struct rrr_instance_thread_data *self
+		struct rrr_instance *self
 );
 int rrr_instance_default_set_output_buffer_ratelimit_when_needed (
 		int *delivery_entry_count,
 		int *delivery_ratelimit_active,
-		struct rrr_instance_thread_data *thread_data
+		struct rrr_instance_runtime_data *thread_data
 );
 
 #endif /* RRR_INSTANCES_H */
