@@ -21,13 +21,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <string.h>
 #include <stdlib.h>
+#include <util/utf8.h>
 
 #include "../log.h"
 
 #include "mqtt_topic.h"
 
-#include "../utf8.h"
-#include "../macro_utils.h"
+#include "../util/macro_utils.h"
 
 struct topic_name_seq {
 	uint32_t c1;
@@ -103,6 +103,10 @@ int rrr_mqtt_topic_validate_name_with_end (
 		const char *topic_name,
 		const char *end
 ) {
+	if (topic_name == end) {
+		return 1;
+	}
+
 	struct topic_name_seq seq = { 0, 0, topic_name };
 
 	return rrr_utf8_validate_and_iterate (
@@ -116,7 +120,11 @@ int rrr_mqtt_topic_validate_name_with_end (
 int rrr_mqtt_topic_validate_name (
 		const char *topic_name
 ) {
-	return rrr_mqtt_topic_validate_name_with_end(
+	if (topic_name == NULL || *topic_name == '\0') {
+		return 1;
+	}
+
+	return rrr_mqtt_topic_validate_name_with_end (
 			topic_name,
 			topic_name + strlen(topic_name)
 	);
@@ -163,6 +171,47 @@ int rrr_mqtt_topic_match_tokens_recursively (
 	return rrr_mqtt_topic_match_tokens_recursively(sub_token->next, pub_token->next);
 }
 
+int rrr_mqtt_topic_match_str_with_end (
+		const char *sub_filter,
+		const char *pub_topic,
+		const char *pub_topic_end
+) {
+	int ret = RRR_MQTT_TOKEN_MISMATCH;
+
+	struct rrr_mqtt_topic_token *sub_filter_tokens = NULL;
+	struct rrr_mqtt_topic_token *pub_topic_tokens = NULL;
+
+	if (rrr_mqtt_topic_tokenize(&sub_filter_tokens, sub_filter) != 0) {
+		RRR_MSG_0("Failed to tokenize filter in rrr_mqtt_topic_match_str\n");
+		ret = RRR_MQTT_TOKEN_INTERNAL_ERROR;
+		goto out;
+	}
+
+	if (rrr_mqtt_topic_tokenize_with_end(&pub_topic_tokens, pub_topic, pub_topic_end) != 0) {
+		RRR_MSG_0("Failed to tokenize topic in rrr_mqtt_topic_match_str\n");
+		ret = RRR_MQTT_TOKEN_INTERNAL_ERROR;
+		goto out;
+	}
+
+	ret = rrr_mqtt_topic_match_tokens_recursively(sub_filter_tokens, pub_topic_tokens);
+
+	out:
+	rrr_mqtt_topic_token_destroy(sub_filter_tokens);
+	rrr_mqtt_topic_token_destroy(pub_topic_tokens);
+	return ret;
+}
+
+int rrr_mqtt_topic_match_str (
+		const char *sub_filter,
+		const char *pub_topic
+) {
+	return rrr_mqtt_topic_match_str_with_end (
+			sub_filter,
+			pub_topic,
+			pub_topic + strlen(pub_topic)
+	);
+}
+
 // Both token trees may contain # and +
 // The master token is usually an ACL entry and the slave a subscription request
 // The # of a slave topic will only match the master topic if the master topic is also # on the same level
@@ -187,7 +236,7 @@ int rrr_mqtt_topic_match_tokens_recursively_acl (
 	else if (*(token_master->data) == '+' || *(token_slave->data) == '+') {
 //		printf ("Preliminary match by slave or master +\n");
 		if (*(token_master->data) == '$') {
-			printf ("Mismatch by master $\n");
+//			printf ("Mismatch by master $\n");
 			return RRR_MQTT_TOKEN_MISMATCH;
 		}
 		// + matches everything on this level, continue
