@@ -24,7 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "net_transport_defines.h"
 
-struct rrr_instance_config;
+struct rrr_instance_config_data;
 
 struct rrr_net_transport_config {
 	char *tls_certificate_file;
@@ -42,7 +42,7 @@ void rrr_net_transport_config_cleanup (
 
 int rrr_net_transport_config_parse (
 		struct rrr_net_transport_config *data,
-		struct rrr_instance_config *config,
+		struct rrr_instance_config_data *config,
 		const char *prefix,
 		int allow_both_transport_type,
 		enum rrr_net_transport_type default_transport
