@@ -23,16 +23,17 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <unistd.h>
 #include <string.h>
 
-#include "posix.h"
 #include "log.h"
 #include "read.h"
 #include "read_constants.h"
-#include "linked_list.h"
-#include "rrr_time.h"
-#include "messages.h"
-#include "message_addr.h"
-#include "message_log.h"
+#include "messages/msg_msg.h"
+#include "messages/msg_addr.h"
+#include "messages/msg_log.h"
 #include "array.h"
+#include "array_tree.h"
+#include "util/posix.h"
+#include "util/linked_list.h"
+#include "util/rrr_time.h"
 
 #define RRR_READ_COLLECTION_CLIENT_TIMEOUT_S 30
 
@@ -48,7 +49,7 @@ struct rrr_read_session *rrr_read_session_new (
 	memset(read_session, '\0', sizeof(*read_session));
 
 	if (src_addr_len > sizeof(read_session->src_addr)) {
-		RRR_BUG("BUG: Address too long (%lu>%lu) in rrr_read_session_new\n", src_addr_len, sizeof(read_session->src_addr));
+		RRR_BUG("BUG: Address too long (%u>%lu) in rrr_read_session_new\n", src_addr_len, sizeof(read_session->src_addr));
 	}
 
 	read_session->last_read_time = rrr_time_get_64();
@@ -151,7 +152,6 @@ int rrr_read_message_using_callbacks (
 		ssize_t read_step_initial,
 		ssize_t read_step_max_size,
 		ssize_t read_max_size,
-		int read_flags,
 		int									 (*function_get_target_size) (
 													struct rrr_read_session *read_session,
 													void *private_arg
@@ -160,10 +160,6 @@ int rrr_read_message_using_callbacks (
 													struct rrr_read_session *read_session,
 													void *private_arg
 											 ),
-		int									 (*function_poll) (
-													int read_flags,
-													void *private_arg
-											 ),
 		int									 (*function_read) (
 													char *buf,
 													ssize_t *read_bytes,
@@ -199,29 +195,23 @@ int rrr_read_message_using_callbacks (
 		goto process_overshoot;
 	}
 
-	if ((ret = function_poll(read_flags, functions_callback_arg)) != RRR_READ_OK) {
-		if (ret == RRR_READ_INCOMPLETE) {
-			if ((read_flags & RRR_READ_F_NO_SLEEPING) == 0) {
-				rrr_posix_usleep(10 * 1000);
-			}
-		}
-		else {
-			RRR_DBG_1("Error from poll callback in rrr_read_message_using_callbacks\n");
-		}
-		goto out;
-	}
-
 	/* Read */
-	ret = function_read (buf, &bytes, read_step_max_size, functions_callback_arg);
-	if (ret != 0) {
-		if (ret == RRR_READ_INCOMPLETE) {
-			goto out;
-		}
-		RRR_DBG_1("Error from read callback in rrr_read_message_using_callbacks\n");
+	int ret_from_read = ret = function_read (buf, &bytes, read_step_max_size, functions_callback_arg);
+
+	// We don't quit on soft error yet, downstream must be able to retrieve the correct read session to
+	// handle errors, which might include to remove the read_session from the collection
+	if (ret & (RRR_READ_HARD_ERROR)) {
+		RRR_MSG_0("Hard error from read callback in rrr_read_message_using_callbacks\n");
 		goto out;
 	}
+	if (ret & RRR_READ_INCOMPLETE) {
+		RRR_BUG("BUG: READ_INCOMPLETE returned from read callback in rrr_read_message_using_callbacks, this is not allowed\n");
+	}
+	if ((ret & RRR_READ_EOF) && bytes != 0) {
+		RRR_BUG("BUG: READ_EOF returned from read callback while bytes was non-zero in rrr_read_message_using_callbacks, this is not allowed\n");
+	}
 
-	/* Check for new read session */
+	/* Check for new read session, this must be done after read */
 	if ((read_session = function_get_read_session(functions_callback_arg)) == NULL) {
 		ret = RRR_READ_HARD_ERROR;
 		goto out;
@@ -236,22 +226,43 @@ int rrr_read_message_using_callbacks (
 	}
 
 	/* Check for EOF / connection close */
-	if (bytes == 0) {
+	if (bytes == 0 || ret_from_read != 0) {
 		// In situations where zero bytes are read, the downstream framework should
 		// return something else than OK. If not, we will always exit here.
 		if (read_session->read_complete_method == RRR_READ_COMPLETE_METHOD_ZERO_BYTES_READ) {
 			if (read_session->target_size > 0) {
 				RRR_BUG("Target size was set in rrr_read_message while complete method was connection closed\n");
 			}
+			RRR_DBG_7("Read returned 0, set target size to bytes read as instructed.\n");
 			read_session->target_size = read_session->rx_buf_wpos;
+			ret = RRR_READ_OK;
+			// Don't goto out, call complete handler after storing buffer
+		}
+		else if (ret_from_read & RRR_READ_EOF) {
+			if (read_session->eof_ok_now && read_session->rx_buf_ptr == NULL && read_session->rx_overshoot == NULL) {
+				// Complete callback says that EOF is OK now
+				RRR_DBG_7("Read returned 0, possible close of connection or EOF. EOF was expected.\n");
+				ret = RRR_READ_EOF;
+			}
+			else {
+				// Unexpected EOF
+				RRR_DBG_7("Read returned 0, possible close of connection or EOF. EOF was NOT expected.\n");
+				ret = RRR_READ_SOFT_ERROR;
+			}
+			goto out;
+		}
+		else if (ret_from_read != 0) {
+			ret = ret_from_read;
+			goto out;
 		}
 		else {
-			RRR_DBG_3("Read returned 0 in rrr_read_message_using_callbacks, possible close of connection\n");
-			ret = RRR_READ_SOFT_ERROR;
+			ret = RRR_READ_INCOMPLETE;
 			goto out;
 		}
 	}
 
+	read_session->eof_ok_now = 0;
+
 	process_overshoot:
 	if (read_session->rx_buf_ptr == NULL) {
 		if (read_session->rx_overshoot != NULL) {
@@ -332,7 +343,7 @@ int rrr_read_message_using_callbacks (
 				RRR_BUG("read_session rx_data_pos out of range after get_target_size in rrr_read_message_using_callbacks\n");
 			}
 
-			RRR_DBG_1("Aligning buffer, skipping %li bytes while reading from socket\n", read_session->rx_buf_skip);
+			RRR_DBG_7("Aligning buffer, skipping %li bytes while reading from socket\n", read_session->rx_buf_skip);
 
 			char *new_buf = malloc(read_session->rx_buf_size);
 			if (new_buf == NULL) {
@@ -380,7 +391,7 @@ int rrr_read_message_using_callbacks (
 		if (function_complete_callback != NULL) {
 			ret = function_complete_callback (read_session, functions_callback_arg);
 			if (ret != 0) {
-				RRR_MSG_0("Error %i from complete callback in rrr_read_message_using_callbacks\n", ret);
+				RRR_DBG_3("Note: Return %i from complete callback in rrr_read_message_using_callbacks\n", ret);
 				goto out;
 			}
 
@@ -410,49 +421,49 @@ int rrr_read_common_receive_message_raw_callback (
 		ssize_t data_size,
 		struct rrr_read_common_receive_message_callback_data *callback_data
 ) {
-	struct rrr_socket_msg *socket_msg = *data;
+	struct rrr_msg *msg = *data;
 
 	int ret = 0;
 
 	// Header CRC32 is checked when reading the data from remote and getting size
-	if (rrr_socket_msg_head_to_host_and_verify(socket_msg, data_size) != 0) {
+	if (rrr_msg_head_to_host_and_verify(msg, data_size) != 0) {
 		RRR_MSG_0("Message was invalid in rrr_socket_common_receive_message_raw_callback\n");
 		ret = RRR_READ_SOFT_ERROR;
 		goto out;
 	}
 
-	if (rrr_socket_msg_check_data_checksum_and_length(socket_msg, data_size) != 0) {
+	if (rrr_msg_check_data_checksum_and_length(msg, data_size) != 0) {
 		RRR_MSG_0 ("Message checksum was invalid in rrr_socket_common_receive_message_raw_callback\n");
 		ret = RRR_READ_SOFT_ERROR;
 		goto out;
 	}
 
-	if (RRR_SOCKET_MSG_IS_RRR_MESSAGE(socket_msg)) {
+	if (RRR_MSG_IS_RRR_MESSAGE(msg)) {
 		if (callback_data->callback_msg == NULL) {
-			RRR_MSG_0("Received an rrr_message in rrr_read_common_receive_message_raw_callback but no callback is defined for this type\n");
+			RRR_MSG_0("Received an rrr_msg_msg in rrr_read_common_receive_message_raw_callback but no callback is defined for this type\n");
 			ret = RRR_READ_SOFT_ERROR;
 			goto out;
 		}
 
-		struct rrr_message *message = (struct rrr_message *) socket_msg;
-		if (rrr_message_to_host_and_verify(message, data_size) != 0) {
+		struct rrr_msg_msg *message = (struct rrr_msg_msg *) msg;
+		if (rrr_msg_msg_to_host_and_verify(message, data_size) != 0) {
 			RRR_MSG_0("Message verification failed in read_message_raw_callback (size: %u<>%u)\n",
 					MSG_TOTAL_SIZE(message), message->msg_size);
 			ret = RRR_READ_SOFT_ERROR;
 			goto out;
 		}
 
-		ret = callback_data->callback_msg((struct rrr_message **) data, callback_data->callback_arg);
+		ret = callback_data->callback_msg((struct rrr_msg_msg **) data, callback_data->callback_arg);
 	}
-	else if (RRR_SOCKET_MSG_IS_RRR_MESSAGE_ADDR(socket_msg)) {
+	else if (RRR_MSG_IS_RRR_MESSAGE_ADDR(msg)) {
 		if (callback_data->callback_addr_msg == NULL) {
-			RRR_MSG_0("Received an rrr_message_addr in rrr_read_common_receive_message_raw_callback but no callback is defined for this type\n");
+			RRR_MSG_0("Received an rrr_msg_addr in rrr_read_common_receive_message_raw_callback but no callback is defined for this type\n");
 			ret = RRR_READ_SOFT_ERROR;
 			goto out;
 		}
 
-		struct rrr_message_addr *message = (struct rrr_message_addr *) socket_msg;
-		if (rrr_message_addr_to_host(message) != 0) {
+		struct rrr_msg_addr *message = (struct rrr_msg_addr *) msg;
+		if (rrr_msg_addr_to_host(message) != 0) {
 			RRR_MSG_0("Invalid data in received address message in rrr_read_common_receive_message_raw_callback\n");
 			ret = RRR_READ_SOFT_ERROR;
 			goto out;
@@ -460,15 +471,15 @@ int rrr_read_common_receive_message_raw_callback (
 
 		ret = callback_data->callback_addr_msg(message, callback_data->callback_arg);
 	}
-	else if (RRR_SOCKET_MSG_IS_RRR_MESSAGE_LOG(socket_msg)) {
+	else if (RRR_MSG_IS_RRR_MESSAGE_LOG(msg)) {
 		if (callback_data->callback_log_msg == NULL) {
-			RRR_MSG_0("Received an rrr_message_log in rrr_read_common_receive_message_raw_callback but no callback is defined for this type\n");
+			RRR_MSG_0("Received an rrr_msg_msg_log in rrr_read_common_receive_message_raw_callback but no callback is defined for this type\n");
 			ret = RRR_READ_SOFT_ERROR;
 			goto out;
 		}
 
-		struct rrr_message_log *message = (struct rrr_message_log *) socket_msg;
-		if (rrr_message_log_to_host(message) != 0) {
+		struct rrr_msg_log *message = (struct rrr_msg_log *) msg;
+		if (rrr_msg_msg_log_to_host(message) != 0) {
 			RRR_MSG_0("Invalid data in received log message in rrr_read_common_receive_message_raw_callback\n");
 			ret = RRR_READ_SOFT_ERROR;
 			goto out;
@@ -478,7 +489,7 @@ int rrr_read_common_receive_message_raw_callback (
 	}
 	else {
 		RRR_MSG_0("Received a socket message of unknown type %u in rrr_read_common_receive_message_raw_callback\n",
-				socket_msg->msg_type);
+				msg->msg_type);
 		ret = RRR_READ_SOFT_ERROR;
 		goto out;
 	}
@@ -518,9 +529,9 @@ int rrr_read_common_get_session_target_length_from_message_and_checksum_raw (
 	*result = 0;
 
 	rrr_length target_size = 0;
-	int ret = rrr_socket_msg_get_target_size_and_check_checksum(
+	int ret = rrr_msg_get_target_size_and_check_checksum(
 			&target_size,
-			(struct rrr_socket_msg *) data,
+			(struct rrr_msg *) data,
 			data_size
 	);
 
@@ -568,54 +579,66 @@ int rrr_read_common_get_session_target_length_from_message_and_checksum (
 	return ret;
 }
 
-int rrr_read_common_get_session_target_length_from_array (
+static int __rrr_read_common_get_session_target_length_from_array_tree_callback (
+		struct rrr_array *array, void *arg
+) {
+	struct rrr_read_common_get_session_target_length_from_array_tree_data *data = arg;
+
+	rrr_array_clear(data->array_final);
+	RRR_LL_MERGE_AND_CLEAR_SOURCE_HEAD(data->array_final, array);
+
+	return 0;
+}
+
+int rrr_read_common_get_session_target_length_from_array_tree (
 		struct rrr_read_session *read_session,
 		void *arg
 ) {
-	struct rrr_read_common_get_session_target_length_from_array_data *data = arg;
-
-	if (data->definition == NULL || RRR_LL_COUNT(data->definition) == 0) {
-		RRR_BUG("NULL or empty array definition given to rrr_read_common_get_session_target_length_from_array\n");
-	}
+	struct rrr_read_common_get_session_target_length_from_array_tree_data *data = arg;
 
+	const char *pos_max = (data->message_max_size != 0 ? read_session->rx_buf_ptr + data->message_max_size : NULL);
 	char *pos = read_session->rx_buf_ptr;
 	rrr_slength wpos = read_session->rx_buf_wpos;
 
 //	printf ("Array wpos: %li\n", wpos);
 
-	if (data->message_max_size != 0 && wpos > data->message_max_size) {
-		RRR_DBG_1("Received message exceeds maximum size, is a delimeter missing? (%" PRIrrrsl ">%li)\n",
-				wpos, data->message_max_size);
-		return RRR_READ_SOFT_ERROR;
-	}
-
 	ssize_t import_length = 0;
 	ssize_t skipped_bytes = 0;
 
 	while (wpos > 0) {
-		int ret = rrr_array_get_packed_length_from_buffer (
+		if (pos_max != NULL && pos > pos_max) {
+			RRR_DBG_1("Received array data exceeds maximum size, is a delimeter missing? (%" PRIrrrsl ">%u)\n",
+					wpos, data->message_max_size);
+			return RRR_READ_SOFT_ERROR;
+		}
+
+		int ret = rrr_array_tree_import_from_buffer (
 				&import_length,
-				data->definition,
 				pos,
-				wpos
+				wpos,
+				data->tree,
+				__rrr_read_common_get_session_target_length_from_array_tree_callback,
+				data
 		);
 
 		if (ret == 0) {
+			if (import_length <= 0) {
+				RRR_MSG_0("Warning: Array definition produced a length of zero, possible configuration error. Check REWIND usage.\n");
+				return RRR_READ_SOFT_ERROR;
+			}
 			break;
 		}
 		else {
-			if (ret == RRR_TYPE_PARSE_INCOMPLETE) {
-				return RRR_READ_INCOMPLETE;
+			if (ret == RRR_TYPE_PARSE_SOFT_ERR) {
+				if (data->do_byte_by_byte_sync != 0) {
+					skipped_bytes++;
+					pos++;
+					wpos--;
+					ret = RRR_READ_OK;
+				}
 			}
 
-			if (data->do_byte_by_byte_sync != 0) {
-				skipped_bytes++;
-				pos++;
-				wpos--;
-			}
-			else {
-				return RRR_READ_SOFT_ERROR;
-			}
+			return ret;
 		}
 	}
 
@@ -629,5 +652,9 @@ int rrr_read_common_get_session_target_length_from_array (
 	// Read position for array framework
 	read_session->rx_buf_skip = skipped_bytes;
 
+	if (read_session->target_size == read_session->rx_buf_wpos) {
+		read_session->eof_ok_now = 1;
+	}
+
 	return RRR_READ_OK;
 }
