@@ -31,16 +31,16 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "stats_engine.h"
 #include "stats_message.h"
-
-#include "../gnu.h"
-#include "../socket/rrr_socket.h"
-#include "../socket/rrr_socket_msg.h"
-#include "../socket/rrr_socket_client.h"
 #include "../read.h"
-#include "../linked_list.h"
-#include "../rrr_time.h"
 #include "../random.h"
-#include "../macro_utils.h"
+#include "../socket/rrr_socket.h"
+#include "../socket/rrr_socket_client.h"
+#include "../messages/msg.h"
+#include "../util/rrr_time.h"
+#include "../util/linked_list.h"
+#include "../util/gnu.h"
+#include "../util/macro_utils.h"
+#include "../util/posix.h"
 
 #define RRR_STATS_ENGINE_SEND_INTERVAL_MS 50
 #define RRR_STATS_ENGINE_LOG_JOURNAL_MAX_ENTRIES 25
@@ -51,11 +51,37 @@ struct rrr_stats_client {
 	int first_log_journal_messages_sent;
 };
 
-static void __rrr_stats_engine_unlock_void (void *arg) {
-	pthread_mutex_t *lock = arg;
-	pthread_mutex_unlock(lock);
+static void __rrr_stats_engine_journal_lock (struct rrr_stats_engine *engine) {
+	pthread_mutex_lock(&engine->journal_lock);
+	engine->journal_lock_usercount++;
+	if (engine->journal_lock_usercount > 2) {
+		RRR_BUG("BUG: Stats engine journal lock usercount was > 2\n");
+	}
 }
 
+static void __rrr_stats_engine_journal_unlock (struct rrr_stats_engine *engine) {
+	engine->journal_lock_usercount--;
+	int usercount_now = engine->journal_lock_usercount;
+
+	pthread_mutex_unlock(&engine->journal_lock);
+
+	if (usercount_now < 0) {
+		RRR_BUG("BUG: Stats engine journal lock usercount was < 0\n");
+	}
+}
+
+static void __rrr_stats_engine_journal_unlock_void (void *arg) {
+	struct rrr_stats_engine *engine = arg;
+	__rrr_stats_engine_journal_unlock(engine);
+}
+
+#define JOURNAL_LOCK(engine) 												\
+	__rrr_stats_engine_journal_lock(engine);								\
+	pthread_cleanup_push(__rrr_stats_engine_journal_unlock_void, engine)		\
+
+#define JOURNAL_UNLOCK()													\
+	pthread_cleanup_pop(1)
+
 static void __rrr_stats_engine_log_listener (
 		unsigned short loglevel_translated,
 		const char *prefix,
@@ -72,8 +98,12 @@ static void __rrr_stats_engine_log_listener (
 	(void)(loglevel_translated);
 	(void)(prefix);
 
-	pthread_mutex_lock(&stats->journal_lock);
-	pthread_cleanup_push(__rrr_stats_engine_unlock_void, &stats->journal_lock);
+	JOURNAL_LOCK(stats);
+
+	if (stats->journal_lock_usercount > 1) {
+		// Prevent log loops when sending log messages generates new messages when debug is active
+		goto out;
+	}
 
 	size_t msg_size = strlen(message) + 1;
 
@@ -99,13 +129,16 @@ static void __rrr_stats_engine_log_listener (
 	new_message = NULL;
 
 	out:
-	pthread_cleanup_pop(1);
+	JOURNAL_UNLOCK();
 	if (message != NULL) {
 		rrr_stats_message_destroy(new_message);
 	}
 }
 
-static int __rrr_stats_client_new (struct rrr_stats_client **target, struct rrr_stats_engine *engine) {
+static int __rrr_stats_client_new (
+		struct rrr_stats_client **target,
+		struct rrr_stats_engine *engine
+) {
 	struct rrr_stats_client *client = malloc(sizeof(*client));
 	if (client == NULL) {
 		RRR_MSG_0("Could not allocate memory in __rrr_stats_client_new\n");
@@ -121,7 +154,9 @@ static int __rrr_stats_client_new (struct rrr_stats_client **target, struct rrr_
 	return 0;
 }
 
-static void __rrr_stats_client_destroy (struct rrr_stats_client *client) {
+static void __rrr_stats_client_destroy (
+		struct rrr_stats_client *client
+) {
 	free(client);
 }
 
@@ -133,17 +168,23 @@ static void __rrr_stats_client_destroy_void (void *client) {
 	__rrr_stats_client_destroy (client);
 }
 
-static int __rrr_stats_named_message_list_destroy (struct rrr_stats_named_message_list *list) {
+static int __rrr_stats_named_message_list_destroy (
+		struct rrr_stats_named_message_list *list
+) {
 	RRR_LL_DESTROY(list, struct rrr_stats_message, rrr_stats_message_destroy(node));
 	free(list);
 	return 0;
 }
 
-static void __rrr_stats_named_message_list_collection_clear (struct rrr_stats_named_message_list_collection *collection) {
+static void __rrr_stats_named_message_list_collection_clear (
+		struct rrr_stats_named_message_list_collection *collection
+) {
 	RRR_LL_DESTROY(collection, struct rrr_stats_named_message_list, __rrr_stats_named_message_list_destroy(node));
 }
 
-static void __rrr_stats_log_journal_clear (struct rrr_stats_log_journal *collection) {
+static void __rrr_stats_log_journal_clear (
+		struct rrr_stats_log_journal *collection
+) {
 	RRR_LL_DESTROY(collection, struct rrr_stats_message, rrr_stats_message_destroy(node));
 }
 
@@ -164,16 +205,18 @@ static struct rrr_stats_named_message_list *__rrr_stats_named_message_list_get (
 }
 
 // To provide memory fence, this must be called prior to any thread starting or forking
-int rrr_stats_engine_init (struct rrr_stats_engine *stats) {
+int rrr_stats_engine_init (
+		struct rrr_stats_engine *stats
+) {
 	int ret = 0;
 	char *filename = NULL;
 
 	memset (stats, '\0', sizeof(*stats));
 
-	if (pthread_mutex_init(&stats->main_lock, 0) != 0) {
+	if (rrr_posix_mutex_init(&stats->main_lock, 0) != 0) {
 		RRR_MSG_0("Could not initialize main mutex in rrr_stats_engine_init\n");
 		ret = 1;
-		goto out;
+		goto out_final;
 	}
 
 	pid_t pid = getpid();
@@ -197,7 +240,7 @@ int rrr_stats_engine_init (struct rrr_stats_engine *stats) {
 		goto out_close_socket;
 	}
 
-	if (pthread_mutex_init(&stats->journal_lock, 0) != 0) {
+	if (rrr_posix_mutex_init(&stats->journal_lock, RRR_POSIX_MUTEX_IS_RECURSIVE) != 0) {
 		RRR_MSG_0("Could not initialize journal mutex in rrr_stats_engine_init\n");
 		ret = 1;
 		goto out_destroy_client_collection;
@@ -222,7 +265,7 @@ int rrr_stats_engine_init (struct rrr_stats_engine *stats) {
 			filename, stats->log_hook_handle);
 	stats->initialized = 1;
 
-	goto out;
+	goto out_final;
 	out_destroy_journal_lock:
 		pthread_mutex_destroy(&stats->journal_lock);
 	out_destroy_client_collection:
@@ -231,12 +274,14 @@ int rrr_stats_engine_init (struct rrr_stats_engine *stats) {
 		rrr_socket_close(stats->socket);
 	out_destroy_mutex:
 		pthread_mutex_destroy(&stats->main_lock);
-	out:
+	out_final:
 		RRR_FREE_IF_NOT_NULL(filename);
 		return ret;
 }
 
-void rrr_stats_engine_cleanup (struct rrr_stats_engine *stats) {
+void rrr_stats_engine_cleanup (
+		struct rrr_stats_engine *stats
+) {
 	if (stats->initialized == 0) {
 		return;
 	}
@@ -265,7 +310,10 @@ struct rrr_stats_engine_read_callback_data {
 	struct rrr_stats_engine *engine;
 };
 
-static int __rrr_stats_engine_read_callback (struct rrr_read_session *read_session, void *arg) {
+static int __rrr_stats_engine_read_callback (
+		struct rrr_read_session *read_session,
+		void *arg
+) {
 	struct rrr_stats_engine *stats = arg;
 
 	(void)(stats);
@@ -277,7 +325,7 @@ static int __rrr_stats_engine_read_callback (struct rrr_read_session *read_sessi
 }
 
 int __rrr_stats_engine_multicast_send_intermediate (
-		struct rrr_socket_msg *data,
+		struct rrr_msg *data,
 		size_t size,
 		void *callback_arg
 ) {
@@ -290,8 +338,8 @@ int __rrr_stats_engine_multicast_send_intermediate (
 	);
 }
 
-int __rrr_stats_engine_unicast_send_intermediate (
-		struct rrr_socket_msg *data,
+static int __rrr_stats_engine_unicast_send_intermediate (
+		struct rrr_msg *data,
 		size_t size,
 		void *callback_arg
 ) {
@@ -313,7 +361,7 @@ int __rrr_stats_engine_unicast_send_intermediate (
 static int __rrr_stats_engine_pack_message (
 		const struct rrr_stats_message *message,
 		int (*callback)(
-				struct rrr_socket_msg *data,
+				struct rrr_msg *data,
 				size_t size,
 				void *callback_arg
 		),
@@ -328,15 +376,15 @@ static int __rrr_stats_engine_pack_message (
 			message
 	);
 
-	rrr_socket_msg_populate_head (
-			(struct rrr_socket_msg *) &message_packed,
-			RRR_SOCKET_MSG_TYPE_TREE_DATA,
+	rrr_msg_populate_head (
+			(struct rrr_msg *) &message_packed,
+			RRR_MSG_TYPE_TREE_DATA,
 			total_size,
 			message->timestamp
 	);
 
-	rrr_socket_msg_checksum_and_to_network_endian (
-			(struct rrr_socket_msg *) &message_packed
+	rrr_msg_checksum_and_to_network_endian (
+			(struct rrr_msg *) &message_packed
 	);
 
 	// This is very noisy, disable. Causes self-genration of messages
@@ -347,10 +395,13 @@ static int __rrr_stats_engine_pack_message (
 			message->path
 	);*/
 
-	return callback((struct rrr_socket_msg *) &message_packed, total_size, callback_arg);
+	return callback((struct rrr_msg *) &message_packed, total_size, callback_arg);
 }
 
-static int __rrr_stats_engine_send_messages_from_list (struct rrr_stats_engine *stats, struct rrr_stats_named_message_list *list) {
+static int __rrr_stats_engine_send_messages_from_list (
+		struct rrr_stats_engine *stats,
+		struct rrr_stats_named_message_list *list
+) {
 	int ret = 0;
 
 	int has_clients = (rrr_socket_client_collection_count(&stats->client_collection) > 0 ? 1 : 0);
@@ -393,7 +444,9 @@ static int __rrr_stats_engine_send_messages_from_list (struct rrr_stats_engine *
 	return ret;
 }
 
-static int __rrr_stats_engine_send_messages (struct rrr_stats_engine *stats) {
+static int __rrr_stats_engine_send_messages (
+		struct rrr_stats_engine *stats
+) {
 	int ret = 0;
 
 	RRR_LL_ITERATE_BEGIN(&stats->named_message_list, struct rrr_stats_named_message_list);
@@ -407,7 +460,9 @@ static int __rrr_stats_engine_send_messages (struct rrr_stats_engine *stats) {
 	return ret;
 }
 
-static void __rrr_stats_engine_journal_send_to_new_clients (struct rrr_stats_engine *stats) {
+static void __rrr_stats_engine_journal_send_to_new_clients (
+		struct rrr_stats_engine *stats
+) {
 	// All messages will be sent by the normal function the first iteration
 	if (stats->log_journal_last_sent_message == NULL) {
 		return;
@@ -419,9 +474,7 @@ static void __rrr_stats_engine_journal_send_to_new_clients (struct rrr_stats_eng
 
 		if (client->first_log_journal_messages_sent == 0) {
 			// If journal is busy, just don't do anything.
-			pthread_mutex_lock(&stats->journal_lock);
-
-			pthread_cleanup_push(__rrr_stats_engine_unlock_void, &stats->journal_lock);
+			JOURNAL_LOCK(stats);
 
 			// If there are more than 100 messages in the queue, clean up by removing entries from the beginning
 			RRR_LL_ITERATE_BEGIN(&stats->log_journal, struct rrr_stats_message);
@@ -442,14 +495,16 @@ static void __rrr_stats_engine_journal_send_to_new_clients (struct rrr_stats_eng
 				}
 			RRR_LL_ITERATE_END();
 
-			pthread_cleanup_pop(1);
+			JOURNAL_UNLOCK();
 
 			client->first_log_journal_messages_sent = 1;
 		}
 	RRR_LL_ITERATE_END();
 }
 
-static void __rrr_stats_engine_log_journal_send_to_clients (struct rrr_stats_engine *stats) {
+static void __rrr_stats_engine_log_journal_send_to_clients (
+		struct rrr_stats_engine *stats
+) {
 	int start_sending = 0;
 
 	if (stats->log_journal_last_sent_message == NULL) {
@@ -458,8 +513,7 @@ static void __rrr_stats_engine_log_journal_send_to_clients (struct rrr_stats_eng
 
 	again:
 
-	pthread_mutex_lock(&stats->journal_lock);
-	pthread_cleanup_push(__rrr_stats_engine_unlock_void, &stats->journal_lock);
+	JOURNAL_LOCK(stats);
 
 	int max = 100;
 
@@ -488,7 +542,7 @@ static void __rrr_stats_engine_log_journal_send_to_clients (struct rrr_stats_eng
 		}
 	RRR_LL_ITERATE_END();
 
-	pthread_cleanup_pop(1);
+	JOURNAL_UNLOCK();
 
 	if (start_sending == 0) {
 		start_sending = 1;
@@ -496,9 +550,10 @@ static void __rrr_stats_engine_log_journal_send_to_clients (struct rrr_stats_eng
 	}
 }
 
-static void __rrr_stats_engine_log_journal_trim (struct rrr_stats_engine *stats) {
-	pthread_mutex_lock(&stats->journal_lock);
-	pthread_cleanup_push(__rrr_stats_engine_unlock_void, &stats->journal_lock);
+static void __rrr_stats_engine_log_journal_trim (
+		struct rrr_stats_engine *stats
+) {
+	JOURNAL_LOCK(stats);
 
 	RRR_LL_ITERATE_BEGIN(&stats->log_journal, struct rrr_stats_message);
 		if (RRR_LL_COUNT(&stats->log_journal) > RRR_STATS_ENGINE_LOG_JOURNAL_MAX_ENTRIES) {
@@ -509,10 +564,12 @@ static void __rrr_stats_engine_log_journal_trim (struct rrr_stats_engine *stats)
 		}
 	RRR_LL_ITERATE_END_CHECK_DESTROY(&stats->log_journal, rrr_stats_message_destroy(node));
 
-	pthread_cleanup_pop(1);
+	JOURNAL_UNLOCK();
 }
 
-static void __rrr_stats_engine_log_journal_tick (struct rrr_stats_engine *stats) {
+static void __rrr_stats_engine_log_journal_tick (
+		struct rrr_stats_engine *stats
+) {
 	// New clients receive the full journal up to last_message mark
 	__rrr_stats_engine_journal_send_to_new_clients(stats);
 
@@ -523,7 +580,9 @@ static void __rrr_stats_engine_log_journal_tick (struct rrr_stats_engine *stats)
 	__rrr_stats_engine_log_journal_trim(stats);
 }
 
-int rrr_stats_engine_tick (struct rrr_stats_engine *stats) {
+int rrr_stats_engine_tick (
+		struct rrr_stats_engine *stats
+) {
 	int ret = 0;
 
 	if (stats->initialized == 0) {
@@ -550,10 +609,9 @@ int rrr_stats_engine_tick (struct rrr_stats_engine *stats) {
 	// Read from clients
 	if ((ret = rrr_socket_client_collection_read (
 			&stats->client_collection,
-			sizeof(struct rrr_socket_msg),
+			sizeof(struct rrr_msg),
 			1024,
-			0,
-			RRR_SOCKET_READ_METHOD_RECVFROM,
+			RRR_SOCKET_READ_METHOD_RECVFROM | RRR_SOCKET_READ_CHECK_POLLHUP,
 			rrr_read_common_get_session_target_length_from_message_and_checksum,
 			NULL,
 			__rrr_stats_engine_read_callback,
@@ -611,15 +669,15 @@ static int __rrr_stats_engine_message_register_nolock (
 	if (list == NULL) {
 		RRR_MSG_0("List with handle %u not found in __rrr_stats_engine_message_register_nolock\n", stats_handle);
 		ret = 1;
-		goto out;
+		goto out_final;
 	}
 
-	struct rrr_stats_message *new_message;
+	struct rrr_stats_message *new_message = NULL;
 
 	if (rrr_stats_message_duplicate(&new_message, message) != 0) {
 		RRR_MSG_0("Could not duplicate message in __rrr_stats_engine_message_register_nolock\n");
 		ret = 1;
-		goto out;
+		goto out_final;
 	}
 
 	if (RRR_STATS_MESSAGE_FLAGS_IS_STICKY(new_message)) {
@@ -630,27 +688,30 @@ static int __rrr_stats_engine_message_register_nolock (
 	if (ret >= RRR_STATS_MESSAGE_PATH_MAX_LENGTH) {
 		RRR_MSG_0("Path was too long in __rrr_stats_engine_message_register_nolock\n");
 		ret = 1;
-		goto out;
+		goto out_free;
 	}
 	ret = 0;
 
 	if (rrr_stats_message_set_path(new_message, prefix_tmp) != 0) {
 		RRR_MSG_0("Could not set path in new message in __rrr_stats_engine_message_register_nolock\n");
 		ret = 1;
-		goto out;
+		goto out_free;
 	}
 
 	RRR_LL_APPEND(list, new_message);
 	new_message = NULL;
 
-	out:
-	if (new_message != NULL) {
+	goto out_final;
+	out_free:
 		rrr_stats_message_destroy(new_message);
-	}
-	return ret;
+	out_final:
+		return ret;
 }
 
-static int __rrr_stats_engine_handle_exists_nolock (struct rrr_stats_engine *stats, unsigned int stats_handle) {
+static int __rrr_stats_engine_handle_exists_nolock (
+		struct rrr_stats_engine *stats,
+		unsigned int stats_handle
+) {
 	RRR_LL_ITERATE_BEGIN(&stats->named_message_list, struct rrr_stats_named_message_list);
 		if (node->owner_handle == stats_handle) {
 			return 1;
@@ -659,7 +720,10 @@ static int __rrr_stats_engine_handle_exists_nolock (struct rrr_stats_engine *sta
 	return 0;
 }
 
-static void __rrr_stats_engine_handle_unregister_nolock (struct rrr_stats_engine *stats, unsigned int stats_handle) {
+static void __rrr_stats_engine_handle_unregister_nolock (
+		struct rrr_stats_engine *stats,
+		unsigned int stats_handle
+) {
 	int did_unregister = 0;
 
 	RRR_LL_ITERATE_BEGIN(&stats->named_message_list, struct rrr_stats_named_message_list);
@@ -675,7 +739,10 @@ static void __rrr_stats_engine_handle_unregister_nolock (struct rrr_stats_engine
 	}
 }
 
-static int __rrr_stats_engine_handle_register_nolock (struct rrr_stats_engine *stats, unsigned int stats_handle) {
+static int __rrr_stats_engine_handle_register_nolock (
+		struct rrr_stats_engine *stats,
+		unsigned int stats_handle
+) {
 	int ret = 0;
 
 	struct rrr_stats_named_message_list *entry = malloc(sizeof(*entry));
