@@ -26,8 +26,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <inttypes.h>
 #include <sys/socket.h>
 
-#include "../linked_list.h"
 #include "../socket/rrr_socket_client.h"
+#include "../util/linked_list.h"
 #include "stats_message.h"
 
 #define RRR_STATS_ENGINE_STICKY_SEND_INTERVAL_MS 1000
@@ -52,7 +52,11 @@ struct rrr_stats_engine {
 	int socket;
 	int log_hook_handle;
 	pthread_mutex_t main_lock;
+
+	// Access through macro only to update usercount
 	pthread_mutex_t journal_lock;
+	int journal_lock_usercount;
+
 	struct rrr_stats_named_message_list_collection named_message_list;
 	struct rrr_socket_client_collection client_collection;
 	uint64_t next_send_time;
