@@ -27,21 +27,22 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <inttypes.h>
 
 #include "../lib/log.h"
+#include "../lib/poll_helper.h"
+#include "../lib/instance_config.h"
+#include "../lib/instances.h"
+#include "../lib/threads.h"
+#include "../lib/message_broker.h"
+#include "../lib/array.h"
 #include "../lib/http/http_client.h"
 #include "../lib/http/http_client_config.h"
 #include "../lib/http/http_query_builder.h"
 #include "../lib/http/http_session.h"
 #include "../lib/net_transport/net_transport_config.h"
-#include "../lib/ip_buffer_entry.h"
-#include "../lib/ip_buffer_entry_struct.h"
-#include "../lib/ip_buffer_entry_collection.h"
-#include "../lib/poll_helper.h"
-#include "../lib/instance_config.h"
-#include "../lib/instances.h"
-#include "../lib/messages.h"
-#include "../lib/threads.h"
-#include "../lib/message_broker.h"
-#include "../lib/array.h"
+#include "../lib/net_transport/net_transport.h"
+#include "../lib/messages/msg_msg.h"
+#include "../lib/message_holder/message_holder.h"
+#include "../lib/message_holder/message_holder_struct.h"
+#include "../lib/message_holder/message_holder_collection.h"
 
 #define RRR_HTTPCLIENT_DEFAULT_SERVER			"localhost"
 #define RRR_HTTPCLIENT_DEFAULT_PORT				0 // 0=automatic
@@ -49,19 +50,26 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define RRR_HTTPCLIENT_LIMIT_REDIRECTS_MAX		500
 
 struct httpclient_data {
-	struct rrr_instance_thread_data *thread_data;
+	struct rrr_instance_runtime_data *thread_data;
 	struct rrr_http_client_data http_client_data;
-	struct rrr_ip_buffer_entry_collection defer_queue;
+	struct rrr_msg_holder_collection defer_queue;
 
 	int do_no_data;
 	int do_rrr_msg_to_array;
 	int do_drop_on_error;
+	int do_keepalive;
+	int do_receive_raw_data;
+	int do_send_raw_data;
+
 	rrr_setting_uint message_timeout_us;
 
 	rrr_setting_uint redirects_max;
 
 	struct rrr_net_transport_config net_transport_config;
 
+	struct rrr_net_transport *keepalive_transport;
+	int keepalive_handle;
+
 	// Array fields, server name etc.
 	struct rrr_http_client_config http_client_config;
 };
@@ -87,7 +95,7 @@ static int httpclient_send_request_callback (
 		RRR_BUG("BUG: Invalid response %i propagated from http framework to httpclient module\n", response_code);
 	}
 
-	RRR_DBG_1("HTTP response from server in httpclient instance %s: %i %s\n",
+	RRR_DBG_3("HTTP response from server in httpclient instance %s: %i %s\n",
 			INSTANCE_D_NAME(httpclient_data->thread_data),
 			response_code,
 			(response_argument != NULL ? response_argument : "(no response string)")
@@ -216,6 +224,15 @@ static int httpclient_session_add_fields_callback (
 		RRR_BUG("BUG: HTTP do_no_data is set but tags map and array are not empty in httpclient_session_add_fields_callback\n");
 	}
 
+	if ((ret = rrr_http_session_set_keepalive (
+			session,
+			data->do_keepalive
+	)) != 0) {
+		RRR_MSG_0("Failed to set keep-alive in httpclient_session_add_fields_callback\n");
+		ret = 1;
+		goto out;
+	}
+
 	if (RRR_MAP_COUNT(&data->http_client_config.tags) == 0) {
 		// Add all array fields
 		RRR_LL_ITERATE_BEGIN(callback_data->array, const struct rrr_type_value);
@@ -309,7 +326,7 @@ static int httpclient_reset_client_data (
 static int httpclient_get_values_from_message (
 		struct rrr_array *target_array,
 		struct httpclient_data *data,
-		const struct rrr_message *message
+		const struct rrr_msg_msg *message
 ) {
 	int ret = 0;
 
@@ -362,26 +379,103 @@ static int httpclient_get_values_from_message (
 	return ret;
 }
 
+struct httpclient_raw_create_message_callback_data {
+	struct httpclient_data *httpclient_data;
+	const char *data;
+	ssize_t data_size;
+	const struct rrr_msg_msg *msg_orig;
+};
+
+static int httpclient_raw_create_message_callback (
+		struct rrr_msg_holder *new_entry,
+		void *arg
+) {
+	struct httpclient_raw_create_message_callback_data *callback_data = arg;
+
+	int ret = RRR_MESSAGE_BROKER_OK;
+
+	rrr_biglength size = callback_data->data_size;
+
+	if (size > 0xffffffff) { // Eight f's
+		RRR_MSG_0("HTTP length too long in httpclient_raw_create_message_callback, max is 0xffffffff\n");
+		ret = RRR_MESSAGE_BROKER_DROP;
+		goto out;
+	}
+
+	if ((ret = rrr_msg_msg_new_with_data (
+			(struct rrr_msg_msg **) &new_entry->message,
+			MSG_TYPE_MSG,
+			MSG_CLASS_DATA,
+			rrr_time_get_64(),
+			MSG_TOPIC_PTR(callback_data->msg_orig),
+			MSG_TOPIC_LENGTH(callback_data->msg_orig),
+			callback_data->data,
+			size
+	)) != 0) {
+		RRR_MSG_0("Failed to create message in httpclient_raw_create_message_callback\n");
+		ret = RRR_MESSAGE_BROKER_ERR;
+		goto out;
+	}
+
+	RRR_DBG_3("httpclient instance %s created raw httpclient data message with data size %" PRIrrrbl "\n",
+			INSTANCE_D_NAME(callback_data->httpclient_data->thread_data), size);
+
+	out:
+	rrr_msg_holder_unlock(new_entry);
+	return ret;
+}
+
+struct httpclient_raw_callback_data {
+	struct httpclient_data *httpclient_data;
+	const struct rrr_msg_msg *msg_orig;
+};
+
+static int httpclient_raw_callback (
+		RRR_HTTP_SESSION_RAW_RECEIVE_CALLBACK_ARGS
+)  {
+	struct httpclient_raw_callback_data *callback_data = arg;
+	struct httpclient_data *httpclient_data = callback_data->httpclient_data;
+
+	(void)(unique_id);
+
+	int ret = 0;
+
+	struct httpclient_raw_create_message_callback_data callback_data_broker = {
+			httpclient_data,
+			data,
+			data_size,
+			callback_data->msg_orig
+	};
+
+	return rrr_message_broker_write_entry (
+			INSTANCE_D_BROKER_ARGS(httpclient_data->thread_data),
+			NULL,
+			0,
+			0,
+			httpclient_raw_create_message_callback,
+			&callback_data_broker
+	);
+
+	return ret;
+}
+
 static int httpclient_send_request_locked (
 		struct httpclient_data *data,
-		struct rrr_ip_buffer_entry *entry
+		struct rrr_msg_holder *entry
 ) {
-	struct rrr_message *message = entry->message;
+	struct rrr_msg_msg *message = entry->message;
 	struct rrr_array array_tmp = {0};
 
 	array_tmp.version = RRR_ARRAY_VERSION;
 
 	int ret = RRR_HTTP_OK;
 
-	RRR_DBG_3("httpclient instance %s sending message with timestamp %" PRIu64 "\n",
-			INSTANCE_D_NAME(data->thread_data), message->timestamp);
-
-	if ((ret = httpclient_reset_client_data(data)) != RRR_HTTP_OK) {
-		goto out;
+	if (data->do_keepalive == 0 || data->keepalive_handle == 0) {
+		if ((ret = httpclient_reset_client_data(data)) != RRR_HTTP_OK) {
+			goto out;
+		}
 	}
 
-	(void)(message);
-
 	if (data->message_timeout_us != 0) {
 		if (rrr_time_get_64() > entry->send_time + data->message_timeout_us) {
 			RRR_MSG_0("Send timeout for message in httpclient instance %s, dropping it.\n",
@@ -390,21 +484,35 @@ static int httpclient_send_request_locked (
 		}
 	}
 
-	// If tag filtering is performed, this is done in add_fields_callback. Here, all
-	// values are prepared in the temporary array.
-	if (data->do_no_data == 0) {
-		if ((ret = httpclient_get_values_from_message(&array_tmp, data, message)) != RRR_HTTP_OK) {
+	if (data->do_send_raw_data) {
+		if (MSG_DATA_LENGTH(message) == 0) {
+			RRR_DBG_1("httpclient instance %s has http_send_raw_data set, but a received message had 0 length data. Dropping it.\n",
+					INSTANCE_D_NAME(data->thread_data));
 			goto out;
 		}
+		if (MSG_CLASS(message) != MSG_CLASS_DATA) {
+			RRR_DBG_1("httpclient instance %s has http_send_raw_data set, but a received message had wrong class (%u). Note that only raw data messages can be sent, not arrays.\n",
+					INSTANCE_D_NAME(data->thread_data), MSG_CLASS(message));
+			goto out;
+		}
+	}
+	else {
+		if (data->do_no_data == 0) {
+			// If tag filtering is performed, this is done in add_fields_callback. Here, all
+			// values are prepared in the temporary array.
+			if ((ret = httpclient_get_values_from_message(&array_tmp, data, message)) != RRR_HTTP_OK) {
+				goto out;
+			}
+		}
 	}
 
 	// DO NOT use unsigned here.
-	long long int redirect_retry_max = data->redirects_max;
+	long long int redirect_retry_max = (data->do_send_raw_data ? 0 : data->redirects_max);
 
 	retry:
 
 	if (redirect_retry_max > RRR_HTTPCLIENT_LIMIT_REDIRECTS_MAX || redirect_retry_max < 0) {
-		RRR_BUG("Redirect counter error in httpclient_send_request_locked, value is now %i\n", redirect_retry_max);
+		RRR_BUG("Redirect counter error in httpclient_send_request_locked, value is now %lli\n", redirect_retry_max);
 	}
 
 	struct httpclient_add_fields_callback_data add_fields_callback_data = {
@@ -412,18 +520,55 @@ static int httpclient_send_request_locked (
 		&array_tmp
 	};
 
-	if ((ret = rrr_http_client_send_request (
-			&data->http_client_data,
-			data->http_client_config.method,
-			&data->net_transport_config,
-			httpclient_session_add_fields_callback,
-			&add_fields_callback_data,
-			httpclient_send_request_callback,
-			data
-	)) != RRR_HTTP_OK) {
+	struct httpclient_raw_callback_data raw_callback_data = {
+			data,
+			message
+	};
+
+	if (data->do_send_raw_data) {
+		RRR_DBG_2("httpclient instance %s sending raw request from message with timestamp %" PRIu64 "\n",
+				INSTANCE_D_NAME(data->thread_data), message->timestamp);
+
+		ret = rrr_http_client_send_raw_request (
+				&data->http_client_data,
+				data->http_client_config.method,
+				(data->do_keepalive ? &data->keepalive_transport : NULL),
+				(data->do_keepalive ? &data->keepalive_handle : 0),
+				&data->net_transport_config,
+				MSG_DATA_PTR(message),
+				MSG_DATA_LENGTH(message),
+				(data->do_receive_raw_data ? httpclient_raw_callback : NULL),
+				(data->do_receive_raw_data ? &raw_callback_data : NULL),
+				httpclient_send_request_callback,
+				data
+		);
+	}
+	else {
+		RRR_DBG_2("httpclient instance %s sending request from message with timestamp %" PRIu64 " endpoint %s\n",
+				INSTANCE_D_NAME(data->thread_data), message->timestamp, data->http_client_data.endpoint);
+
+		ret = rrr_http_client_send_request (
+				&data->http_client_data,
+				data->http_client_config.method,
+				(data->do_keepalive ? &data->keepalive_transport : NULL),
+				(data->do_keepalive ? &data->keepalive_handle : 0),
+				&data->net_transport_config,
+				(data->do_receive_raw_data ? httpclient_raw_callback : NULL),
+				(data->do_receive_raw_data ? &raw_callback_data : NULL),
+				httpclient_session_add_fields_callback,
+				&add_fields_callback_data,
+				httpclient_send_request_callback,
+				data
+		);
+	}
+
+	if (ret != RRR_HTTP_OK) {
 		if (ret == RRR_HTTP_SOFT_ERROR) {
 			RRR_DBG_2("HTTP request failed in httpclient instance %s, return was %i\n",
 					INSTANCE_D_NAME(data->thread_data), ret);
+
+			// Make sure new connection is created
+			data->keepalive_handle = 0;
 		}
 		else {
 			RRR_MSG_0("HTTP request failed in httpclient instance %s, return was %i\n",
@@ -459,9 +604,9 @@ static int httpclient_send_request_locked (
 static int httpclient_poll_callback(RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 //	printf ("httpclient got entry %p\n", entry);
 
-	struct rrr_instance_thread_data *thread_data = arg;
+	struct rrr_instance_runtime_data *thread_data = arg;
 	struct httpclient_data *data = thread_data->private_data;
-	struct rrr_message *message = entry->message;
+	struct rrr_msg_msg *message = entry->message;
 
 	RRR_DBG_3("httpclient instance %s received message with timestamp %" PRIu64 "\n",
 			INSTANCE_D_NAME(thread_data), message->timestamp);
@@ -473,9 +618,8 @@ static int httpclient_poll_callback(RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 
 	if ((ret = httpclient_send_request_locked(data, entry)) != 0) {
 		if (ret == RRR_HTTP_SOFT_ERROR) {
-			RRR_MSG_0("Soft error while sending message in httpclient instance %s, deferring message\n",
+			RRR_DBG_1("Soft error while sending message in httpclient instance %s, deferring message\n",
 					INSTANCE_D_NAME(thread_data));
-			ret = 0;
 			goto out_defer;
 		}
 		RRR_MSG_0("Hard error while sending message in httpclient instance %s\n",
@@ -486,26 +630,29 @@ static int httpclient_poll_callback(RRR_MODULE_POLL_CALLBACK_SIGNATURE) {
 
 	goto out;
 	out_defer:
-		rrr_ip_buffer_entry_incref_while_locked(entry);
+		rrr_msg_holder_incref_while_locked(entry);
 		RRR_LL_APPEND(&data->defer_queue, entry);
-		rrr_ip_buffer_entry_unlock(entry);
+		rrr_msg_holder_unlock(entry);
 		return RRR_FIFO_SEARCH_STOP;
 	out:
-		rrr_ip_buffer_entry_unlock(entry);
+		rrr_msg_holder_unlock(entry);
 		return ret;
 }
 
 static void httpclient_data_cleanup(void *arg) {
 	struct httpclient_data *data = arg;
+	if (data->keepalive_transport != NULL) {
+		rrr_net_transport_destroy(data->keepalive_transport);
+	}
 	rrr_http_client_data_cleanup(&data->http_client_data);
 	rrr_net_transport_config_cleanup(&data->net_transport_config);
 	rrr_http_client_config_cleanup(&data->http_client_config);
-	rrr_ip_buffer_entry_collection_clear(&data->defer_queue);
+	rrr_msg_holder_collection_clear(&data->defer_queue);
 }
 
 static int httpclient_data_init (
 		struct httpclient_data *data,
-		struct rrr_instance_thread_data *thread_data
+		struct rrr_instance_runtime_data *thread_data
 ) {
 	int ret = 0;
 
@@ -514,27 +661,30 @@ static int httpclient_data_init (
 	data->thread_data = thread_data;
 
 	if ((ret = rrr_http_client_data_init(&data->http_client_data, RRR_HTTP_CLIENT_USER_AGENT)) != 0) {
-		RRR_MSG_0("Could not initialize httpclient data in httpclient_data_init\n");
+		RRR_MSG_0("Could not initialize httpclient httpclient_data in httpclient_data_init\n");
 		ret = 1;
 		goto out;
 	}
 
 	goto out;
 //	out_cleanup_data:
-//		httpclient_data_cleanup(data);
+//		httpclient_data_cleanup(httpclient_data);
 	out:
 		return ret;
 }
 
 static int httpclient_parse_config (
 		struct httpclient_data *data,
-		struct rrr_instance_config *config
+		struct rrr_instance_config_data *config
 ) {
 	int ret = 0;
 
 	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("http_no_data", do_no_data, 0);
 	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("http_rrr_msg_to_array", do_rrr_msg_to_array, 0);
 	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("http_drop_on_error", do_drop_on_error, 0);
+	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("http_keepalive", do_keepalive, 0);
+	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("http_receive_raw_data", do_receive_raw_data, 0);
+	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("http_send_raw_data", do_send_raw_data, 0);
 
 	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_UNSIGNED("http_message_timeout_ms", message_timeout_us, 0);
 	// Remember to mulitply to get useconds. Zero means no timeout.
@@ -549,19 +699,6 @@ static int httpclient_parse_config (
 		goto out;
 	}
 
-	if (rrr_http_client_config_parse (
-			&data->http_client_config,
-			config,
-			"http",
-			RRR_HTTPCLIENT_DEFAULT_SERVER,
-			RRR_HTTPCLIENT_DEFAULT_PORT,
-			0, // <-- Disable fixed tags and fields
-			1  // <-- Enable endpoint
-	) != 0) {
-		ret = 1;
-		goto out;
-	}
-
 	if (data->do_no_data) {
 		if (RRR_MAP_COUNT(&data->http_client_config.tags) > 0) {
 			RRR_MSG_0("Setting http_no_data in instance %s was 'yes' while http_tags was also set. This is an error.\n",
@@ -578,6 +715,36 @@ static int httpclient_parse_config (
 		}
 	}
 
+	if (data->do_send_raw_data) {
+		if (data->do_no_data) {
+			RRR_MSG_0("Both http_send_raw_data and http_no_data was yes in httpclient instance %s. The first implies the latter, it is an error to specify both.\n",
+					config->name);
+			ret = 1;
+		}
+		if (data->do_rrr_msg_to_array) {
+			RRR_MSG_0("http_rrr_msg_to_array as well as http_send_raw_data were yes in httpclient instance %s, this is an invalid combination.\n",
+								config->name);
+			ret = 1;
+		}
+		if (ret != 0) {
+			goto out;
+		}
+	}
+
+	if (rrr_http_client_config_parse (
+			&data->http_client_config,
+			config,
+			"http",
+			RRR_HTTPCLIENT_DEFAULT_SERVER,
+			RRR_HTTPCLIENT_DEFAULT_PORT,
+			0, // <-- Disable fixed tags and fields
+			1, // <-- Enable endpoint
+			data->do_send_raw_data // Check raw consisitency based on this option
+	) != 0) {
+		ret = 1;
+		goto out;
+	}
+
 	if (rrr_net_transport_config_parse (
 			&data->net_transport_config,
 			config,
@@ -594,7 +761,7 @@ static int httpclient_parse_config (
 }
 
 static void *thread_entry_httpclient (struct rrr_thread *thread) {
-	struct rrr_instance_thread_data *thread_data = thread->private_data;
+	struct rrr_instance_runtime_data *thread_data = thread->private_data;
 	struct httpclient_data *data = thread_data->private_data = thread_data->private_memory;
 
 	if (httpclient_data_init(data, thread_data) != 0) {
@@ -607,7 +774,7 @@ static void *thread_entry_httpclient (struct rrr_thread *thread) {
 	pthread_cleanup_push(httpclient_data_cleanup, data);
 
 	rrr_thread_set_state(thread, RRR_THREAD_STATE_INITIALIZED);
-	rrr_thread_signal_wait(thread_data->thread, RRR_THREAD_SIGNAL_START);
+	rrr_thread_signal_wait(thread, RRR_THREAD_SIGNAL_START);
 	rrr_thread_set_state(thread, RRR_THREAD_STATE_RUNNING);
 
 	if (httpclient_parse_config(data, INSTANCE_D_CONFIG(thread_data)) != 0) {
@@ -616,18 +783,16 @@ static void *thread_entry_httpclient (struct rrr_thread *thread) {
 
 	rrr_instance_config_check_all_settings_used(thread_data->init_data.instance_config);
 
-	rrr_poll_add_from_thread_senders (thread_data->poll, thread_data);
-
 	RRR_DBG_1 ("httpclient started thread %p\n", thread_data);
 
-	while (rrr_thread_check_encourage_stop(thread_data->thread) != 1) {
-		rrr_thread_update_watchdog_time(thread_data->thread);
+	while (rrr_thread_check_encourage_stop(thread) != 1) {
+		rrr_thread_update_watchdog_time(thread);
 
 		if (RRR_LL_COUNT(&data->defer_queue) > 0) {
 			int ret_tmp = RRR_HTTP_OK;
 
-			RRR_LL_ITERATE_BEGIN(&data->defer_queue, struct rrr_ip_buffer_entry);
-				rrr_ip_buffer_entry_lock(node);
+			RRR_LL_ITERATE_BEGIN(&data->defer_queue, struct rrr_msg_holder);
+				rrr_msg_holder_lock(node);
 				if ((ret_tmp = httpclient_send_request_locked(data, node)) != RRR_HTTP_OK) {
 					if (ret_tmp == RRR_HTTP_SOFT_ERROR) {
 						// Let soft error propagate
@@ -642,16 +807,16 @@ static void *thread_entry_httpclient (struct rrr_thread *thread) {
 				else {
 					RRR_LL_ITERATE_SET_DESTROY();
 				}
-				rrr_ip_buffer_entry_unlock(node);
-			RRR_LL_ITERATE_END_CHECK_DESTROY(&data->defer_queue, 0; rrr_ip_buffer_entry_decref(node));
+				rrr_msg_holder_unlock(node);
+			RRR_LL_ITERATE_END_CHECK_DESTROY(&data->defer_queue, 0; rrr_msg_holder_decref(node));
 
 			if (ret_tmp == RRR_HTTP_SOFT_ERROR) {
 				rrr_posix_usleep(500000); // 500ms to avoid spamming server when there are errors
 			}
 		}
 		else {
-			if (rrr_poll_do_poll_delete (thread_data, thread_data->poll, httpclient_poll_callback, 50) != 0) {
-				RRR_MSG_ERR("Error while polling in httpclient instance %s\n",
+			if (rrr_poll_do_poll_delete (thread_data, &thread_data->poll, httpclient_poll_callback, 30) != 0) {
+				RRR_MSG_0("Error while polling in httpclient instance %s\n",
 						INSTANCE_D_NAME(thread_data));
 				break;
 			}
@@ -659,22 +824,16 @@ static void *thread_entry_httpclient (struct rrr_thread *thread) {
 	}
 
 	out_message:
-	RRR_DBG_1 ("Thread httpclient %p exiting\n", thread_data->thread);
+	RRR_DBG_1 ("Thread httpclient %p exiting\n", thread);
 
 	pthread_cleanup_pop(1);
 	pthread_exit(0);
 }
 
-static int test_config (struct rrr_instance_config *config) {
-	RRR_DBG_1("Dummy configuration test for instance %s\n", config->name);
-	return 0;
-}
-
 static struct rrr_module_operations module_operations = {
 		NULL,
 		thread_entry_httpclient,
 		NULL,
-		test_config,
 		NULL,
 		NULL
 };
@@ -684,10 +843,10 @@ static const char *module_name = "httpclient";
 __attribute__((constructor)) void load(void) {
 }
 
-void init(struct rrr_instance_dynamic_data *data) {
+void init(struct rrr_instance_module_data *data) {
 	data->private_data = NULL;
 	data->module_name = module_name;
-	data->type = RRR_MODULE_TYPE_DEADEND;
+	data->type = RRR_MODULE_TYPE_PROCESSOR;
 	data->operations = module_operations;
 	data->dl_ptr = NULL;
 }
