@@ -30,25 +30,29 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../lib/log.h"
 
 #include "../lib/instance_config.h"
-#include "../lib/rrr_time.h"
 #include "../lib/threads.h"
 #include "../lib/instances.h"
-#include "../lib/messages.h"
-#include "../lib/ip.h"
-#include "../lib/ip_buffer_entry.h"
-#include "../lib/ip_buffer_entry_struct.h"
 #include "../lib/message_broker.h"
-#include "../lib/stats/stats_instance.h"
 #include "../lib/random.h"
 #include "../lib/array.h"
-#include "../lib/linked_list.h"
-#include "../lib/gnu.h"
 #include "../lib/rrr_strerror.h"
+#include "../lib/stats/stats_instance.h"
+#include "../lib/messages/msg_msg.h"
+#include "../lib/ip/ip.h"
+#include "../lib/message_holder/message_holder.h"
+#include "../lib/message_holder/message_holder_struct.h"
+#include "../lib/util/linked_list.h"
+#include "../lib/util/gnu.h"
+#include "../lib/util/rrr_time.h"
 
 // No trailing or leading /
 #define RRR_JOURNAL_TOPIC_PREFIX "rrr/journal"
 #define RRR_JOURNAL_HOSTNAME_MAX_LEN 256
 
+// Other threads must sleep when queue is full
+#define RRR_JOURNAL_DELIVERY_QUEUE_SLEEP_LIMIT 5000
+#define RRR_JOURNAL_DELIVERY_QUEUE_SLEEP_TIME_MS 5
+
 struct journal_queue_entry {
 	RRR_LL_NODE(struct journal_queue_entry);
 	uint64_t timestamp;
@@ -60,13 +64,16 @@ struct journal_queue {
 };
 
 struct journal_data {
-	struct rrr_instance_thread_data *thread_data;
+	struct rrr_instance_runtime_data *thread_data;
 
 	int do_generate_test_messages;
+
 	int log_hook_handle;
 
 	pthread_mutex_t delivery_lock;
 	struct journal_queue delivery_queue;
+	uint64_t delivery_queue_sleep_event_count;
+
 	int is_in_hook;
 	int error_in_hook;
 
@@ -99,7 +106,7 @@ static void journal_queue_entry_destroy (struct journal_queue_entry *node) {
 	free(node);
 }
 
-static int journal_data_init(struct journal_data *data, struct rrr_instance_thread_data *thread_data) {
+static int journal_data_init(struct journal_data *data, struct rrr_instance_runtime_data *thread_data) {
 
 	// memset 0 is done in preload function, DO NOT do that here
 
@@ -120,7 +127,7 @@ static void journal_data_cleanup(void *arg) {
 	pthread_mutex_unlock(&data->delivery_lock);
 }
 
-static int journal_parse_config (struct journal_data *data, struct rrr_instance_config *config) {
+static int journal_parse_config (struct journal_data *data, struct rrr_instance_config_data *config) {
 	int ret = 0;
 
 	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("journal_generate_test_messages", do_generate_test_messages, 0);
@@ -128,8 +135,8 @@ static int journal_parse_config (struct journal_data *data, struct rrr_instance_
 
 	if (data->hostname == NULL || *(data->hostname) == '\0') {
 		char hostname[RRR_JOURNAL_HOSTNAME_MAX_LEN+1];
-		int ret = 0;
-		if ((ret = gethostname(hostname, sizeof(hostname))) != 0) {
+
+		if (gethostname(hostname, sizeof(hostname)) != 0) {
 			RRR_MSG_0("Could not get system hostname in journal instance %s: %s\n",
 					INSTANCE_D_NAME(data->thread_data), rrr_strerror(errno));
 			ret = 1;
@@ -150,32 +157,21 @@ static int journal_parse_config (struct journal_data *data, struct rrr_instance_
 
 // Lock must be initialized before other locks start to provide correct memory fence
 static int journal_preload (struct rrr_thread *thread) {
-	struct rrr_instance_thread_data *thread_data = thread->private_data;
+	struct rrr_instance_runtime_data *thread_data = thread->private_data;
 	struct journal_data *data = thread_data->private_data = thread_data->private_memory;
 
 	int ret = 0;
 
 	memset(data, '\0', sizeof(*data));
 
-	pthread_mutexattr_t attr;
-	if (pthread_mutexattr_init(&attr) != 0) {
-		RRR_MSG_0("Could not initialize mutexattr in journal_preload\n");
+	if (rrr_posix_mutex_init(&data->delivery_lock, RRR_POSIX_MUTEX_IS_RECURSIVE) != 0) {
+		RRR_MSG_0("Could not initialize lock in journal_preload\n");
 		ret = 1;
 		goto out;
 	}
 
-	pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
-
-	if (pthread_mutex_init(&data->delivery_lock, &attr) != 0) {
-		RRR_MSG_0("Could not initialize lock in journal_preload\n");
-		ret = 1;
-		goto out_cleanup_attr;
-	}
-
-	out_cleanup_attr:
-		pthread_mutexattr_destroy(&attr);
 	out:
-		return ret;
+	return ret;
 }
 
 // Note : Context here is ANY thread
@@ -187,9 +183,21 @@ static void journal_log_hook (
 ) {
 	struct journal_data *data = private_arg;
 
+	// Make the calling thread pause a bit to reduce the amount of messages
+	// coming in. This is done if we are unable to handle request due to
+	// slowness of the readers of journal module. DO NOT sleep inside the
+	// lock, that would make things worse by making journal module unable
+	// to empty the delivery queue
+	int do_sleep_before_return = 0;
+
 	// This is a recursive lock
 	pthread_mutex_lock(&data->delivery_lock);
 
+	if (RRR_LL_COUNT(&data->delivery_queue) > RRR_JOURNAL_DELIVERY_QUEUE_SLEEP_LIMIT) {
+		do_sleep_before_return = 1;
+		data->delivery_queue_sleep_event_count++;
+	}
+
 	struct journal_queue_entry *entry = NULL;
 
 	data->count_total++;
@@ -241,17 +249,27 @@ static void journal_log_hook (
 			journal_queue_entry_destroy(entry);
 		}
 		pthread_mutex_unlock(&data->delivery_lock);
+		if (do_sleep_before_return) {
+			rrr_posix_usleep(RRR_JOURNAL_DELIVERY_QUEUE_SLEEP_TIME_MS * 1000);
+		}
 		return;
 }
 
-static int journal_write_message_callback (struct rrr_ip_buffer_entry *entry, void *arg) {
-	struct journal_data *data = arg;
+struct journal_write_message_callback_data {
+	struct journal_data *data;
+	int entry_count;
+	int entry_count_limit;
+};
+
+static int journal_write_message_callback (struct rrr_msg_holder *entry, void *arg) {
+	struct journal_write_message_callback_data *callback_data = arg;
+	struct journal_data *data = callback_data->data;
 
 	int ret = 0;
 
 	char *topic_tmp = NULL;
 	char *topic_tmp_final = NULL;
-	struct rrr_message *reading = NULL;
+	struct rrr_msg_msg *reading = NULL;
 	struct journal_queue_entry *queue_entry = NULL;
 
 	pthread_mutex_lock (&data->delivery_lock);
@@ -307,7 +325,9 @@ static int journal_write_message_callback (struct rrr_ip_buffer_entry *entry, vo
 
 	reading = NULL;
 
-	if (RRR_LL_COUNT(&data->delivery_queue) > 0) {
+	callback_data->entry_count++;
+
+	if (RRR_LL_COUNT(&data->delivery_queue) > 0 && callback_data->entry_count < callback_data->entry_count_limit) {
 		ret = RRR_MESSAGE_BROKER_AGAIN;
 	}
 
@@ -319,7 +339,7 @@ static int journal_write_message_callback (struct rrr_ip_buffer_entry *entry, vo
 	RRR_FREE_IF_NOT_NULL(topic_tmp_final);
 	RRR_FREE_IF_NOT_NULL(topic_tmp);
 	RRR_FREE_IF_NOT_NULL(reading);
-	rrr_ip_buffer_entry_unlock(entry);
+	rrr_msg_holder_unlock(entry);
 	return ret;
 }
 
@@ -337,7 +357,7 @@ static void journal_delivery_lock_cleanup(void *arg) {
 }
 
 static void *thread_entry_journal (struct rrr_thread *thread) {
-	struct rrr_instance_thread_data *thread_data = thread->private_data;
+	struct rrr_instance_runtime_data *thread_data = thread->private_data;
 	struct journal_data *data = thread_data->private_data = thread_data->private_memory;
 
 	// This cleanup must happen after the hook is unregistered
@@ -354,7 +374,7 @@ static void *thread_entry_journal (struct rrr_thread *thread) {
 	pthread_cleanup_push(journal_unregister_handle, data);
 
 	rrr_thread_set_state(thread, RRR_THREAD_STATE_INITIALIZED);
-	rrr_thread_signal_wait(thread_data->thread, RRR_THREAD_SIGNAL_START);
+	rrr_thread_signal_wait(thread, RRR_THREAD_SIGNAL_START);
 	rrr_thread_set_state(thread, RRR_THREAD_STATE_RUNNING);
 
 	if (journal_parse_config(data, thread_data->init_data.instance_config) != 0) {
@@ -379,8 +399,10 @@ static void *thread_entry_journal (struct rrr_thread *thread) {
 
 	uint64_t next_test_msg_time = 0;
 
-	while (!rrr_thread_check_encourage_stop(thread_data->thread)) {
-		rrr_thread_update_watchdog_time(thread_data->thread);
+	uint64_t prev_delivery_queue_sleep_event_count = 0;
+
+	while (!rrr_thread_check_encourage_stop(thread)) {
+		rrr_thread_update_watchdog_time(thread);
 
 		if (data->error_in_hook) {
 			RRR_MSG_0("Error encountered inside log hook of journal instance %s, exiting\n",
@@ -397,6 +419,12 @@ static void *thread_entry_journal (struct rrr_thread *thread) {
 			}
 		}
 
+		struct journal_write_message_callback_data callback_data = {
+			data,
+			0,
+			400
+		};
+
 		if (rrr_message_broker_write_entry (
 				INSTANCE_D_BROKER(thread_data),
 				INSTANCE_D_HANDLE(thread_data),
@@ -404,7 +432,7 @@ static void *thread_entry_journal (struct rrr_thread *thread) {
 				0,
 				0,
 				journal_write_message_callback,
-				data
+				&callback_data
 		)) {
 			RRR_MSG_0("Could not create new message in journal instance %s\n",
 					INSTANCE_D_NAME(thread_data));
@@ -412,17 +440,53 @@ static void *thread_entry_journal (struct rrr_thread *thread) {
 		}
 
 		if (time_now - time_start > 1000000) {
+			int output_buffer_count = 0;
+			int delivery_ratelimit_active = 0;
+			uint64_t delivery_queue_sleep_event_count = 0;
+			int delivery_queue_count = 0;
+
+			if (rrr_instance_default_set_output_buffer_ratelimit_when_needed (
+					&output_buffer_count,
+					&delivery_ratelimit_active,
+					thread_data
+			) != 0) {
+				RRR_MSG_0("Error while setting ratelimit in journal instance %s\n",
+						INSTANCE_D_NAME(thread_data));
+				break;
+			}
+
+			pthread_mutex_lock(&data->delivery_lock);
+			delivery_queue_sleep_event_count = data->delivery_queue_sleep_event_count;
+			delivery_queue_count = RRR_LL_COUNT(&data->delivery_queue);
+			pthread_mutex_unlock(&data->delivery_lock);
+
 			time_start = time_now;
 			rrr_stats_instance_update_rate (INSTANCE_D_STATS(thread_data), 0, "processed", data->count_processed - prev_processed);
 			rrr_stats_instance_update_rate (INSTANCE_D_STATS(thread_data), 1, "suppressed", data->count_suppressed - prev_suppressed);
 			rrr_stats_instance_update_rate (INSTANCE_D_STATS(thread_data), 2, "total", data->count_total - prev_total);
+			rrr_stats_instance_update_rate (INSTANCE_D_STATS(thread_data), 3, "delivery_queue_sleeps", delivery_queue_sleep_event_count - prev_delivery_queue_sleep_event_count);
+			rrr_stats_instance_post_unsigned_base10_text (
+					INSTANCE_D_STATS(thread_data),
+					"output_buffer_count",
+					0,
+					output_buffer_count
+			);
+			rrr_stats_instance_post_unsigned_base10_text (
+					INSTANCE_D_STATS(thread_data),
+					"delivery_queue_count",
+					0,
+					delivery_queue_count
+			);
 
+			prev_delivery_queue_sleep_event_count = delivery_queue_sleep_event_count;
 			prev_processed = data->count_processed;
 			prev_suppressed = data->count_suppressed;
 			prev_total = data->count_total;
 		}
 
-		rrr_posix_usleep (50000); // 50 ms
+		if (callback_data.entry_count == 0) {
+			rrr_posix_usleep (50000); // 50 ms
+		}
 	}
 
 	out_cleanup:
@@ -434,20 +498,10 @@ static void *thread_entry_journal (struct rrr_thread *thread) {
 	pthread_exit(0);
 }
 
-static int test_config (struct rrr_instance_config *config) {
-	struct journal_data data;
-	int ret = 0;
-	memset(&data, '\0', sizeof(data));
-	ret = journal_parse_config(&data, config);
-	journal_data_cleanup(&data);
-	return ret;
-}
-
 static struct rrr_module_operations module_operations = {
 	journal_preload,
 	thread_entry_journal,
 	NULL,
-	test_config,
 	NULL,
 	NULL
 };
@@ -457,7 +511,7 @@ static const char *module_name = "journal";
 __attribute__((constructor)) void load(void) {
 }
 
-void init(struct rrr_instance_dynamic_data *data) {
+void init(struct rrr_instance_module_data *data) {
 		data->module_name = module_name;
 		data->type = RRR_MODULE_TYPE_SOURCE;
 		data->operations = module_operations;
