@@ -31,29 +31,30 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "../lib/log.h"
 #include "../lib/settings.h"
-#include "../lib/rrr_time.h"
 #include "../lib/threads.h"
-#include "../lib/messages.h"
-#include "../lib/socket/rrr_socket.h"
-#include "../lib/socket/rrr_socket_client.h"
 #include "../lib/read.h"
 #include "../lib/array.h"
+#include "../lib/array_tree.h"
 #include "../lib/instances.h"
 #include "../lib/instance_config.h"
-#include "../lib/utf8.h"
 #include "../lib/message_broker.h"
-#include "../lib/ip_buffer_entry.h"
-#include "../lib/ip_buffer_entry_struct.h"
+#include "../lib/messages/msg_msg.h"
+#include "../lib/socket/rrr_socket.h"
+#include "../lib/socket/rrr_socket_client.h"
+#include "../lib/message_holder/message_holder.h"
+#include "../lib/message_holder/message_holder_struct.h"
+#include "../lib/util/utf8.h"
+#include "../lib/util/rrr_time.h"
 
 struct socket_data {
-	struct rrr_instance_thread_data *thread_data;
+	struct rrr_instance_runtime_data *thread_data;
 	char *socket_path;
 	char *default_topic;
 	ssize_t default_topic_length;
 	int receive_rrr_message;
 	int do_sync_byte_by_byte;
 	int do_unlink_if_exists;
-	struct rrr_array definitions;
+	struct rrr_array_tree *tree;
 	struct rrr_socket_client_collection clients;
 	int socket_fd;
 	uint64_t message_count;
@@ -61,13 +62,15 @@ struct socket_data {
 
 void data_cleanup(void *arg) {
 	struct socket_data *data = (struct socket_data *) arg;
-	rrr_array_clear(&data->definitions);
+	if (data->tree != NULL) {
+		rrr_array_tree_destroy(data->tree);
+	}
 	rrr_socket_client_collection_clear(&data->clients);
 	RRR_FREE_IF_NOT_NULL(data->socket_path);
 	RRR_FREE_IF_NOT_NULL(data->default_topic);
 }
 
-int data_init(struct socket_data *data, struct rrr_instance_thread_data *thread_data) {
+int data_init(struct socket_data *data, struct rrr_instance_runtime_data *thread_data) {
 	memset(data, '\0', sizeof(*data));
 
 	data->thread_data = thread_data;
@@ -75,7 +78,7 @@ int data_init(struct socket_data *data, struct rrr_instance_thread_data *thread_
 	return 0;
 }
 
-int parse_config (struct socket_data *data, struct rrr_instance_config *config) {
+int parse_config (struct socket_data *data, struct rrr_instance_config_data *config) {
 	int ret = 0;
 
 	// Socket path
@@ -95,12 +98,12 @@ int parse_config (struct socket_data *data, struct rrr_instance_config *config)
 
 	// Message default topic
 	if ((ret = rrr_settings_get_string_noconvert_silent(&data->default_topic, config->settings, "socket_default_topic")) != 0) {
-		if (ret != RRR_SETTING_NOT_FOUND) {
+		ret &= ~(RRR_SETTING_NOT_FOUND);
+		if (ret != 0) {
 			RRR_MSG_0("Error while parsing configuration parameter socket_default_path in socket instance %s\n", config->name);
 			ret = 1;
 			goto out;
 		}
-		ret = 0;
 	}
 	else {
 		if (rrr_utf8_validate(data->default_topic, strlen(data->default_topic)) != 0) {
@@ -116,16 +119,18 @@ int parse_config (struct socket_data *data, struct rrr_instance_config *config)
 	if (rrr_instance_config_check_yesno (&yesno, config, "socket_receive_rrr_message") == RRR_SETTING_PARSE_ERROR) {
 		RRR_MSG_0 ("mysql: Could not understand argument socket_receive_rrr_message of instance '%s', please specify 'yes' or 'no'\n",
 				config->name);
-		return 1;
+		ret = 1;
+		goto out;
 	}
 	data->receive_rrr_message = (yesno == 0 || yesno == 1 ? yesno : 0);
 
 	// Parse expected input data
 	if (rrr_instance_config_setting_exists(config, "socket_input_types")) {
-		if ((ret = rrr_instance_config_parse_array_definition_from_config_silent_fail(&data->definitions, config, "socket_input_types")) != 0) {
+		if (rrr_instance_config_parse_array_tree_definition_from_config_silent_fail(&data->tree, config, "socket_input_types") != 0) {
 			RRR_MSG_0("Could not parse configuration parameter socket_input_types in socket instance %s\n",
 					config->name);
-			return 1;
+			ret = 1;
+			goto out;
 		}
 	}
 
@@ -138,38 +143,43 @@ int parse_config (struct socket_data *data, struct rrr_instance_config *config)
 			ret = 1;
 			goto out;
 		}
-		ret = 0;
 	}
 	data->do_sync_byte_by_byte = yesno;
 
-	if (data->receive_rrr_message != 0 && RRR_LL_COUNT(&data->definitions) > 0) {
+	if (data->receive_rrr_message != 0 && data->tree != NULL) {
 		RRR_MSG_0("Array definition cannot be specified with socket_input_types while socket_receive_rrr_message is yes in instance %s\n",
 				config->name);
-		return 1;
+		ret = 1;
+		goto out;
 	}
-	else if (data->receive_rrr_message == 0 && RRR_LL_COUNT(&data->definitions) == 0) {
+	else if (data->receive_rrr_message == 0 && data->tree != NULL) {
 		RRR_MSG_0("No data types defined in socket_input_types for instance %s and socket_receive_rrr_message was not 'yes', can't receive anything.\n",
 				config->name);
-		return 1;
+		ret = 1;
+		goto out;
 	}
 
 	RRR_INSTANCE_CONFIG_PARSE_OPTIONAL_YESNO("socket_unlink_if_exists", do_unlink_if_exists, 0);
 
+	// Reset any NOT_FOUND
+	ret = 0;
+
 	out:
 	return ret;
 }
 
 struct read_data_receive_message_callback_data {
 	struct socket_data *data;
-	struct rrr_ip_buffer_entry *entry;
+	struct rrr_msg_holder *entry;
+	struct rrr_array *array_final;
 };
 
-int read_rrr_message_callback (struct rrr_message **message, void *arg) {
+int read_rrr_msg_msg_callback (struct rrr_msg_msg **message, void *arg) {
 	struct read_data_receive_message_callback_data *callback_data = arg;
 	struct socket_data *data = callback_data->data;
 
 	if (MSG_TOPIC_LENGTH(*message) == 0 && data->default_topic != NULL) {
-		if (rrr_message_topic_set(message, data->default_topic, strlen(data->default_topic)) != 0) {
+		if (rrr_msg_msg_topic_set(message, data->default_topic, strlen(data->default_topic)) != 0) {
 			RRR_MSG_0("Could not set topic of message in rread_data_receive_callback of instance %s\n",
 					INSTANCE_D_NAME(data->thread_data));
 			return 1;
@@ -189,19 +199,18 @@ int read_raw_data_callback (struct rrr_read_session *read_session, void *arg) {
 	struct read_data_receive_message_callback_data *callback_data = arg;
 	struct socket_data *data = callback_data->data;
 
+	(void)(read_session);
+
 	int ret = 0;
 
-	struct rrr_message *message = NULL;
+	struct rrr_msg_msg *message = NULL;
 
-	ssize_t parsed_bytes;
-	if ((ret = rrr_array_new_message_from_buffer (
+	if ((ret = rrr_array_new_message_from_collection (
 			&message,
-			&parsed_bytes,
-			read_session->rx_buf_ptr,
-			read_session->rx_buf_wpos,
+			callback_data->array_final,
+			rrr_time_get_64(),
 			data->default_topic,
-			data->default_topic_length,
-			&data->definitions
+			data->default_topic_length
 	)) != 0) {
 		RRR_MSG_0("Could not create array message in read_data_receive_message_callback of socket instance %s\n",
 				INSTANCE_D_NAME(data->thread_data));
@@ -217,29 +226,31 @@ int read_raw_data_callback (struct rrr_read_session *read_session, void *arg) {
 	return ret;
 }
 
-int read_data_receive_callback (struct rrr_ip_buffer_entry *entry, void *arg) {
+int read_data_receive_callback (struct rrr_msg_holder *entry, void *arg) {
 	struct socket_data *data = arg;
 
 	int ret = 0;
 
+	struct rrr_array array_tmp = {0};
+
 	struct read_data_receive_message_callback_data socket_callback_data = {
 			data,
-			entry
+			entry,
+			&array_tmp
 	};
 
 	if (data->receive_rrr_message != 0) {
 		struct rrr_read_common_receive_message_callback_data read_callback_data = {
-				read_rrr_message_callback,
+				read_rrr_msg_msg_callback,
 				NULL,
 				NULL,
 				&socket_callback_data
 		};
 		if ((ret = rrr_socket_client_collection_read (
 				&data->clients,
-				sizeof(struct rrr_socket_msg),
+				sizeof(struct rrr_msg),
 				4096,
-				RRR_READ_F_NO_SLEEPING,
-				RRR_SOCKET_READ_METHOD_RECVFROM,
+				RRR_SOCKET_READ_METHOD_RECVFROM | RRR_SOCKET_READ_CHECK_POLLHUP,
 				rrr_read_common_get_session_target_length_from_message_and_checksum,
 				NULL,
 				rrr_read_common_receive_message_callback,
@@ -249,18 +260,26 @@ int read_data_receive_callback (struct rrr_ip_buffer_entry *entry, void *arg) {
 		}
 	}
 	else {
-		struct rrr_read_common_get_session_target_length_from_array_data callback_data = {
-				&data->definitions,
+		/*
+
+		const struct rrr_array_tree *tree;
+		struct rrr_array *array_final;
+		int do_byte_by_byte_sync;
+		unsigned int message_max_size;
+
+		*/
+		struct rrr_read_common_get_session_target_length_from_array_tree_data callback_data = {
+				data->tree,
+				&array_tmp,
 				data->do_sync_byte_by_byte,
 				0 // TODO : Set max size?
 		};
 		if ((ret = rrr_socket_client_collection_read (
 				&data->clients,
-				sizeof(struct rrr_socket_msg),
+				sizeof(struct rrr_msg),
 				4096,
-				RRR_READ_F_NO_SLEEPING,
-				RRR_SOCKET_READ_METHOD_RECVFROM,
-				rrr_read_common_get_session_target_length_from_array,
+				RRR_SOCKET_READ_METHOD_RECVFROM | RRR_SOCKET_READ_CHECK_POLLHUP,
+				rrr_read_common_get_session_target_length_from_array_tree,
 				&callback_data,
 				read_raw_data_callback,
 				&socket_callback_data
@@ -269,7 +288,7 @@ int read_data_receive_callback (struct rrr_ip_buffer_entry *entry, void *arg) {
 		}
 	}
 
-	struct rrr_message *message = entry->message;
+	struct rrr_msg_msg *message = entry->message;
 
 	if (message == NULL) {
 		ret = RRR_MESSAGE_BROKER_DROP;
@@ -283,7 +302,8 @@ int read_data_receive_callback (struct rrr_ip_buffer_entry *entry, void *arg) {
 	}
 
 	out:
-	rrr_ip_buffer_entry_unlock(entry);
+	rrr_array_clear(&array_tmp);
+	rrr_msg_holder_unlock(entry);
 	return ret;
 }
 
@@ -332,7 +352,7 @@ static void socket_stop (void *arg) {
 }
 
 static void *thread_entry_socket (struct rrr_thread *thread) {
-	struct rrr_instance_thread_data *thread_data = thread->private_data;
+	struct rrr_instance_runtime_data *thread_data = thread->private_data;
 	struct socket_data *data = thread_data->private_data = thread_data->private_memory;
 
 	pthread_cleanup_push(data_cleanup, data);
@@ -349,7 +369,7 @@ static void *thread_entry_socket (struct rrr_thread *thread) {
 	pthread_cleanup_push(socket_stop, data);
 
 	rrr_thread_set_state(thread, RRR_THREAD_STATE_INITIALIZED);
-	rrr_thread_signal_wait(thread_data->thread, RRR_THREAD_SIGNAL_START);
+	rrr_thread_signal_wait(thread, RRR_THREAD_SIGNAL_START);
 	rrr_thread_set_state(thread, RRR_THREAD_STATE_RUNNING);
 
 	if (parse_config(data, thread_data->init_data.instance_config) != 0) {
@@ -370,11 +390,11 @@ static void *thread_entry_socket (struct rrr_thread *thread) {
 			INSTANCE_D_NAME(thread_data), data->socket_path);
 
 	unsigned int consecutive_nothing_happened = 0;
-	while (!rrr_thread_check_encourage_stop(thread_data->thread)) {
-		rrr_thread_update_watchdog_time(thread_data->thread);
+	while (!rrr_thread_check_encourage_stop(thread)) {
+		rrr_thread_update_watchdog_time(thread);
 
 		if (rrr_socket_client_collection_accept_simple(&data->clients) != 0) {
-			RRR_MSG_ERR("Error while accepting connections in socket instance %s\n",
+			RRR_MSG_0("Error while accepting connections in socket instance %s\n",
 				INSTANCE_D_NAME(thread_data));
 			break;
 		}
@@ -385,11 +405,11 @@ static void *thread_entry_socket (struct rrr_thread *thread) {
 			if (err == RRR_SOCKET_SOFT_ERROR) {
 				// Upon receival of invalid data, we must close the socket as sizes of
 				// the messages and boundaries might be out of sync
-				RRR_MSG_ERR("Invalid data received in socket instance %s, socket must be closed\n",
+				RRR_MSG_0("Invalid data received in socket instance %s, socket must be closed\n",
 						INSTANCE_D_NAME(thread_data));
 			}
 			else {
-				RRR_MSG_ERR("Error while reading data in socket instance %s, return was %i\n",
+				RRR_MSG_0("Error while reading data in socket instance %s, return was %i\n",
 						INSTANCE_D_NAME(thread_data), err);
 			}
 			break;
@@ -412,24 +432,10 @@ static void *thread_entry_socket (struct rrr_thread *thread) {
 	pthread_cleanup_pop(1);
 	pthread_exit(0);
 }
-
-static int test_config (struct rrr_instance_config *config) {
-	struct socket_data data;
-	int ret = 0;
-	if ((ret = data_init(&data, NULL)) != 0) {
-		goto err;
-	}
-	ret = parse_config(&data, config);
-	data_cleanup(&data);
-	err:
-	return ret;
-}
-
 static struct rrr_module_operations module_operations = {
 	NULL,
 	thread_entry_socket,
 	NULL,
-	test_config,
 	NULL,
 	NULL
 };
@@ -439,7 +445,7 @@ static const char *module_name = "socket";
 __attribute__((constructor)) void load(void) {
 }
 
-void init(struct rrr_instance_dynamic_data *data) {
+void init(struct rrr_instance_module_data *data) {
 		data->module_name = module_name;
 		data->type = RRR_MODULE_TYPE_SOURCE;
 		data->operations = module_operations;
