@@ -47,19 +47,19 @@ Modified to fit 2-channel device with unitversion == 5 && subtype == 7.
 
 #include "../lib/log.h"
 #include "../lib/instance_config.h"
-#include "../lib/rrr_time.h"
 #include "../lib/threads.h"
 #include "../lib/instances.h"
-#include "../lib/messages.h"
 #include "../lib/array.h"
-#include "../lib/ip.h"
-#include "../lib/ip_buffer_entry.h"
-#include "../lib/ip_buffer_entry_struct.h"
 #include "../lib/array.h"
 #include "../lib/message_broker.h"
+#include "../lib/messages/msg_msg.h"
+#include "../lib/ip/ip.h"
+#include "../lib/message_holder/message_holder.h"
+#include "../lib/message_holder/message_holder_struct.h"
+#include "../lib/util/rrr_time.h"
 
 struct voltmonitor_data {
-	struct rrr_instance_thread_data *thread_data;
+	struct rrr_instance_runtime_data *thread_data;
 
 #ifdef RRR_WITH_USB
 	usb_dev_handle *usb_handle;
@@ -308,7 +308,7 @@ static int usb_read_voltage(struct voltmonitor_data *data, int *millivolts) {
 }
 #endif
 
-int data_init(struct voltmonitor_data *data, struct rrr_instance_thread_data *thread_data) {
+int data_init(struct voltmonitor_data *data, struct rrr_instance_runtime_data *thread_data) {
 	memset(data, '\0', sizeof(*data));
 	data->thread_data = thread_data;
 	return 0;
@@ -341,7 +341,7 @@ int convert_integer_10(const char *value, int *result) {
 	return 0;
 }
 
-int parse_config(struct voltmonitor_data *data, struct rrr_instance_config *config) {
+int parse_config(struct voltmonitor_data *data, struct rrr_instance_config_data *config) {
 	int ret = 0;
 
 	char *vm_calibration = NULL;
@@ -406,12 +406,12 @@ struct volmonitor_spawn_message_callback_data {
 	uint64_t time_now;
 };
 
-static int voltmonitor_spawn_message_callback (struct rrr_ip_buffer_entry *entry, void *arg) {
+static int voltmonitor_spawn_message_callback (struct rrr_msg_holder *entry, void *arg) {
 	struct volmonitor_spawn_message_callback_data *callback_data = arg;
 
 	int ret = 0;
 
-	struct rrr_message *message = NULL;
+	struct rrr_msg_msg *message = NULL;
 
 	if (rrr_array_new_message_from_collection (
 			&message,
@@ -432,7 +432,7 @@ static int voltmonitor_spawn_message_callback (struct rrr_ip_buffer_entry *entry
 	message = NULL;
 
 	out:
-	rrr_ip_buffer_entry_unlock(entry);
+	rrr_msg_holder_unlock(entry);
 	RRR_FREE_IF_NOT_NULL(message);
 	return ret;
 }
@@ -494,10 +494,10 @@ static int voltmonitor_spawn_test_messages (struct voltmonitor_data *data) {
 	return ret;
 }
 
-int inject (struct rrr_instance_thread_data *thread_data, struct rrr_ip_buffer_entry *entry) {
+int inject (struct rrr_instance_runtime_data *thread_data, struct rrr_msg_holder *entry) {
 	struct voltmonitor_data *data = thread_data->private_data = thread_data->private_memory;
 
-	struct rrr_message *message = entry->message;
+	struct rrr_msg_msg *message = entry->message;
 
 	int ret = 0;
 
@@ -524,16 +524,14 @@ int inject (struct rrr_instance_thread_data *thread_data, struct rrr_ip_buffer_e
 	}
 
 	rrr_array_clear(&array_tmp);
-	rrr_ip_buffer_entry_unlock(entry);
+	rrr_msg_holder_unlock(entry);
 	return ret;
 }
 
 static void *thread_entry_voltmonitor (struct rrr_thread *thread) {
-	struct rrr_instance_thread_data *thread_data = thread->private_data;
+	struct rrr_instance_runtime_data *thread_data = thread->private_data;
 	struct voltmonitor_data *data = thread_data->private_data = thread_data->private_memory;
 
-	thread_data->thread = thread;
-
 	if (data_init(data, thread_data) != 0) {
 		RRR_MSG_0("Could not initialize data in voltmonitor instance %s\n", INSTANCE_D_NAME(thread_data));
 		pthread_exit(0);
@@ -549,7 +547,7 @@ static void *thread_entry_voltmonitor (struct rrr_thread *thread) {
 //	pthread_cleanup_push(rrr_thread_set_stopping, thread);
 
 	rrr_thread_set_state(thread, RRR_THREAD_STATE_INITIALIZED);
-	rrr_thread_signal_wait(thread_data->thread, RRR_THREAD_SIGNAL_START);
+	rrr_thread_signal_wait(thread, RRR_THREAD_SIGNAL_START);
 	rrr_thread_set_state(thread, RRR_THREAD_STATE_RUNNING);
 
 	if (parse_config(data, thread_data->init_data.instance_config) != 0) {
@@ -568,8 +566,8 @@ static void *thread_entry_voltmonitor (struct rrr_thread *thread) {
 		}
 	}
 
-	while (!rrr_thread_check_encourage_stop(thread_data->thread)) {
-		rrr_thread_update_watchdog_time(thread_data->thread);
+	while (!rrr_thread_check_encourage_stop(thread)) {
+		rrr_thread_update_watchdog_time(thread);
 
 		int millivolts;
 		if (usb_read_voltage(data, &millivolts) != 0) {
@@ -580,7 +578,7 @@ static void *thread_entry_voltmonitor (struct rrr_thread *thread) {
 
 		if (!data->do_inject_only) {
 			if (voltmonitor_spawn_message(data, abs(millivolts)) != 0) {
-				RRR_MSG_ERR("Error when spawning message in averager instance %s\n",
+				RRR_MSG_0("Error when spawning message in averager instance %s\n",
 						INSTANCE_D_NAME(thread_data));
 				break;
 			}
@@ -599,26 +597,10 @@ static void *thread_entry_voltmonitor (struct rrr_thread *thread) {
 	pthread_exit(0);
 }
 
-static int test_config (struct rrr_instance_config *config) {
-	struct voltmonitor_data data;
-	int ret = 0;
-
-	if ((ret = data_init(&data, NULL)) != 0) {
-		goto err;
-	}
-
-	ret = parse_config(&data, config);
-	data_cleanup(&data);
-
-	err:
-	return ret;
-}
-
 static struct rrr_module_operations module_operations = {
 		NULL,
 		thread_entry_voltmonitor,
 		NULL,
-		test_config,
 		inject,
 		NULL
 };
@@ -631,7 +613,7 @@ __attribute__((constructor)) void load(void) {
 #endif
 }
 
-void init(struct rrr_instance_dynamic_data *data) {
+void init(struct rrr_instance_module_data *data) {
 		data->module_name = module_name;
 		data->type = RRR_MODULE_TYPE_SOURCE;
 		data->operations = module_operations;
