@@ -5,9 +5,12 @@ TEST_VARIABLES_SET=1
 SHELL_VARS_SET=1
 RRR_WITH_SYSTEMD=yes
 RRR_WITH_PYTHON3=yes
-RRR_WITH_OPENSSL=yes
 RRR_WITH_PERL5=yes
 RRR_WITH_MYSQL=yes
+RRR_WITH_OPENSSL=yes
+RRR_WITH_OPENSSL_CRYPT=
+RRR_WITH_LIBRESSL=no
+RRR_WITH_AUTO_SSL=yes
 
 TEST_COMPOSITE_FILE=./.test_composite.sh
 VARIABLES_FILE=../../variables.sh
@@ -31,6 +34,9 @@ RRR_POST_ARGS="$TEST_SOCKET -d 8 -f - -c 1 -a be4#int1,be3#int2,be2s#int3,be1#in
 SEND_IP=./send_ip
 SEND_IP_PORT=2222
 
+SEND_FIFO=./send_fifo
+SEND_FIFO_FILENAME=/tmp/rrr_test_fifo.sock
+
 fail () {
 	echo "Test $1 failed" 1>&2
 	exit 1
@@ -50,6 +56,12 @@ do_send_ip() {
 	$SEND_IP $SEND_IP_PORT < $TEST_DATA_FILE
 }
 
+do_send_fifo() {
+	echo \$ $SEND_FIFO $SEND_FIFO_FILENAME \< $TEST_DATA_FILE
+	$SEND_FIFO $SEND_FIFO_FILENAME < $TEST_DATA_FILE
+}
+
+
 print_test_header () {
 	echo "============================================="
 	echo "== Test $1 $2"
@@ -86,7 +98,17 @@ do_test_ip() {
 	if test $? -ne 0; then
 		fail $conf 
 	fi
-	
+}
+
+do_test_fifo() {
+	conf=$1
+	print_test_header FIFO $conf
+ 	do_send_fifo &
+	echo \$ $TEST $conf
+	$TEST $conf
+	if test $? -ne 0; then
+		fail $conf 
+	fi
 }
 
 $TEST_DATA_GENERATOR $TEST_DATA_FILE
@@ -102,6 +124,8 @@ do_test_simple test_journal.conf
 
 do_test_ip test_ip.conf
 
+do_test_fifo test_fifo.conf
+
 do_test_socket test_socket.conf
 do_test_socket test_http.conf
 do_test_socket test_ipclient.conf
